@@ -22,8 +22,20 @@
  * Kent, Ohio  44240
  */
 
+/*
+ * xerr messages Copyright (C) 1989-2021  Alan R. Baldwin
+ * from ASxxxx 5.40
+ * Things missing that exist in 5.40:
+ * .rel format
+ * cycle counting
+ */
+
 /*
  * Extensions: P. Felber
+ *
+ * they seem to be:
+ * ldhl
+ * lda
  */
 
 /* Gameboy mods by Roger Ivie (ivie at cc dot usu dot edu); see gb.h for more info 
@@ -32,7 +44,7 @@
 #include "asxxxx.h"
 #include "gb.h"
 
-char    *cpu    = "GameBoy Z80-like CPU";
+char    *cpu    = "GameBoy";
 char    *dsft   = "asm";
 
 char    imtab[3] = { 0x46, 0x56, 0x5E };
@@ -49,6 +61,56 @@ char    imtab[3] = { 0x46, 0x56, 0x5E };
 #define UN      ((char) (OPCY_NONE | 0x00))
 #define P2      ((char) (OPCY_NONE | 0x01))
 
+/*
+ * GB Opcode Cycle Pages
+ */
+
+static char  gbpg1[256] = {
+/*--*--* 0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F */
+/*--*--* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
+/*00*/   4,12, 8, 8, 4, 4, 8, 4,20, 8, 8, 8, 4, 4, 8, 4,
+/*10*/   4,12, 8, 8, 4, 4, 8, 4,12, 8, 8, 8, 4, 4, 8, 4,
+/*20*/  12,12, 8, 8, 4, 4, 8, 4,12, 8, 8, 8, 4, 4, 8, 4,
+/*30*/  12,12, 8, 8,12,12,12, 4,12, 8, 8, 8, 4, 4, 8, 4,
+/*40*/   4, 4, 4, 4, 4, 4, 8, 4, 4, 4, 4, 4, 4, 4, 8, 4,
+/*50*/   4, 4, 4, 4, 4, 4, 8, 4, 4, 4, 4, 4, 4, 4, 8, 4,
+/*60*/   4, 4, 4, 4, 4, 4, 8, 4, 4, 4, 4, 4, 4, 4, 8, 4,
+/*70*/   8, 8, 8, 8, 8, 8, 4, 8, 4, 4, 4, 4, 4, 4, 8, 4,
+/*80*/   4, 4, 4, 4, 4, 4, 8, 4, 4, 4, 4, 4, 4, 4, 8, 4,
+/*90*/   4, 4, 4, 4, 4, 4, 8, 4, 4, 4, 4, 4, 4, 4, 8, 4,
+/*A0*/   4, 4, 4, 4, 4, 4, 8, 4, 4, 4, 4, 4, 4, 4, 8, 4,
+/*B0*/   4, 4, 4, 4, 4, 4, 8, 4, 4, 4, 4, 4, 4, 4, 8, 4,
+/*C0*/  20,12,16,16,24,16, 8,16,20,16,16,P2,24,24, 8,16,
+/*D0*/  20,12,16,UN,24,16, 8,16,20,16,16,UN,24,UN, 8,16,
+/*E0*/  12,12, 8,UN,UN,16, 8,16,16, 4,16,UN,UN,UN, 8,16,
+/*F0*/  12,12, 8, 4,UN,16, 8,16,12, 8,16, 4,UN,UN, 8,16
+};
+
+static char  gbpg2[256] = {  /* P2 == CB */
+/*--*--* 0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F */
+/*--*--* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
+/*00*/   8, 8, 8, 8, 8, 8,16, 8, 8, 8, 8, 8, 8, 8,16, 8,
+/*10*/   8, 8, 8, 8, 8, 8,16, 8, 8, 8, 8, 8, 8, 8,16, 8,
+/*20*/   8, 8, 8, 8, 8, 8,16, 8, 8, 8, 8, 8, 8, 8,16, 8,
+/*30*/   8, 8, 8, 8, 8, 8,16, 8, 8, 8, 8, 8, 8, 8,16, 8,
+/*40*/   8, 8, 8, 8, 8, 8,12, 8, 8, 8, 8, 8, 8, 8,12, 8,
+/*50*/   8, 8, 8, 8, 8, 8,12, 8, 8, 8, 8, 8, 8, 8,12, 8,
+/*60*/   8, 8, 8, 8, 8, 8,12, 8, 8, 8, 8, 8, 8, 8,12, 8,
+/*70*/   8, 8, 8, 8, 8, 8,12, 8, 8, 8, 8, 8, 8, 8,12, 8,
+/*80*/   8, 8, 8, 8, 8, 8,16, 8, 8, 8, 8, 8, 8, 8,16, 8,
+/*90*/   8, 8, 8, 8, 8, 8,16, 8, 8, 8, 8, 8, 8, 8,16, 8,
+/*A0*/   8, 8, 8, 8, 8, 8,16, 8, 8, 8, 8, 8, 8, 8,16, 8,
+/*B0*/   8, 8, 8, 8, 8, 8,16, 8, 8, 8, 8, 8, 8, 8,16, 8,
+/*C0*/   8, 8, 8, 8, 8, 8,16, 8, 8, 8, 8, 8, 8, 8,16, 8,
+/*D0*/   8, 8, 8, 8, 8, 8,16, 8, 8, 8, 8, 8, 8, 8,16, 8,
+/*E0*/   8, 8, 8, 8, 8, 8,16, 8, 8, 8, 8, 8, 8, 8,16, 8,
+/*F0*/   8, 8, 8, 8, 8, 8,16, 8, 8, 8, 8, 8, 8, 8,16, 8
+};
+
+static char *gbPage[2] = {
+    gbpg1, gbpg2
+};
+
 /*
  * Process a machine op.
  */
@@ -76,7 +138,7 @@ struct mne *mp;
                         if ((v1 = admode(CND)) != 0) {
                                 outab(op | (v1<<3));
                         } else {
-                                qerr();
+                                xerr('q', "Condition codes are NZ, Z, NC, and C.");
                         }
                 } else {
                         outab(0xC9);
@@ -92,13 +154,13 @@ struct mne *mp;
                         outab(op | (v1<<4));
                         break;
                 }
-                aerr();
+                xerr('a', "Register SP is invalid.");
                 break;
 
         case S_RST:
                 v1 = (int) absexpr();
                 if (v1 & ~0x38) {
-                        aerr();
+                        xerr('a', "Valid values are N * 0x08, N = 0 -> 7.");
                         v1 = 0;
                 }
                 outab(op|v1);
@@ -108,7 +170,7 @@ struct mne *mp;
                 expr(&e1, 0);
                 abscheck(&e1);
                 if (e1.e_addr > 2) {
-                        aerr();
+                        xerr('a', "Valid values are 0 -> 2.");
                         e1.e_addr = 0;
                 }
                 outab(op);
@@ -128,7 +190,7 @@ struct mne *mp;
                 addr(&e2);
                 abscheck(&e1);
                 if (genop(0xCB, op, &e2, 0) || t1)
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 break;
 
         case S_RL:
@@ -142,8 +204,10 @@ struct mne *mp;
                         t2 = addr(&e2);
                 }
                 if (genop(0xCB, op, &e2, 0) || t1)
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 break;
+        
+        /* TODO: where is S_SWAP? */
 
         case S_AND:
         case S_SUB:
@@ -157,7 +221,7 @@ struct mne *mp;
                         t2 = addr(&e2);
                 }
                 if (genop(0, op, &e2, 1) || t1)
-                        aerr();
+                        xerr('a', "Invalid Addressing Mode.");
                 break;
 
         case S_ADC:
@@ -170,15 +234,15 @@ struct mne *mp;
                 }
                 if (t2 == 0) {
                         if (genop(0, op, &e1, 1))
-                                aerr();
+                                xerr('a', "Invalid Addressing Mode.");
                         break;
                 }
                 if ((t1 == S_R8) && (e1.e_addr == A)) {
                         if (genop(0, op, &e2, 1))
-                                aerr();
+                                xerr('a', "Invalid Addressing Mode.");
                         break;
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
 
         case S_ADD:
@@ -190,32 +254,40 @@ struct mne *mp;
                 }
                 if (t2 == 0) {
                         if (genop(0, op, &e1, 1))
-                                aerr();
+                                xerr('a', "Invalid Addressing Mode.");
                         break;
                 }
                 if ((t1 == S_R8) && (e1.e_addr == A)) {
                         if (genop(0, op, &e2, 1))
-                                aerr();
+                                xerr('a', "Invalid Addressing Mode.");
                         break;
                 }
                 if ((t1 == S_R16) && (t2 == S_R16)) {
-                        op = 0x09;
-                        v1 = (int) e1.e_addr;
-                        v2 = (int) e2.e_addr;
-                        if ((v1 == HL) && (v2 <= SP)) {
-                                outab(op | (v2 << 4));
-                                break;
+                        if( rf == S_ADD ) {
+                                op = 0x09;
+                                v1 = (int) e1.e_addr;
+                                v2 = (int) e2.e_addr;
+                                if ((v1 == HL) && (v2 <= SP)) {
+                                        outab(op | (v2 << 4));
+                                        break;
+                                }
                         }
                 }
                 /*
                  * 0xE8 : ADD SP,#n
                  */
-                if ((t1 == S_R16) && (e1.e_addr == SP) && (t2 == S_IMMED)) {
-                        outab(0xE8);
-                        outrb(&e2,0);
-                        break;
+                if ((t1 == S_R16) && (t2 == S_IMMED)) {
+                        if (rf != S_ADD ) {
+                                xerr('a', "ADC, SUB, and SBC are invalid.");
+                                break;
+                        }
+                        if( e1.e_addr == SP ) {
+                                outab(0xE8);
+                                outrb(&e2,0);
+                                break;
+                        }
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
 
         case S_LD:
@@ -319,9 +391,10 @@ struct mne *mp;
                         outab(0x22);
                         break;
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
 
+        /* TODO: find out where this handled in upstream */
         case S_STOP:    /* 0x10 */
                 /*
                  * 0x10 : STOP
@@ -330,8 +403,7 @@ struct mne *mp;
                 outab(0x00);
                 break;
 
-
-        case S_LDA:     /* 0xE8 */
+        case S_LDA:     /* 0xE8 */ /* extension */
                 /*
                  * 0xE8 : LDA SP,#n(SP)
                  * 0xF8 : LDA HL,#n(SP)
@@ -349,10 +421,10 @@ struct mne *mp;
                         outrb(&e2,0);
                         break;
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
 
-        case S_LDHL:    /* 0xF8 */
+        case S_LDHL:    /* 0xF8 */ /* extension */
                 /*
                  * 0xF8 : LDHL SP,#n
                  */
@@ -364,7 +436,7 @@ struct mne *mp;
                         outrb(&e2,0);
                         break;
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
 
         case S_DEC:
@@ -389,7 +461,7 @@ struct mne *mp;
                                 break;
                         }
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
 
         case S_JR:
@@ -397,7 +469,7 @@ struct mne *mp;
                         if ((v1 &= 0xFF) <= 0x03) {
                                 op += (v1+1)<<3;
                         } else {
-                                aerr();
+                                xerr('q', "Condition codes are NZ, Z, NC, and C.");
                         }
                         comma(1);
                 }
@@ -406,7 +478,7 @@ struct mne *mp;
                 if (mchpcr(&e2)) {
                         v2 = (int) (e2.e_addr - dot.s_addr - 1);
                         if (pass == 2 && ((v2 < -128) || (v2 > 127)))
-                                aerr();
+                                xerr('a', "Branching Range Exceeded.");
                         outab(v2);
                 } else {
                         outrb(&e2, R_PCR);
@@ -446,7 +518,7 @@ struct mne *mp;
                         outab(0xE9);
                         break;
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
 
         case S_LDH:
@@ -481,7 +553,7 @@ struct mne *mp;
                         outab(0xE2);
                         break;
                 }
-                aerr();
+                xerr('a', "Invalid Addressing Mode.");
                 break;
 
 	case S_TILE:
@@ -499,8 +571,7 @@ struct mne *mp;
 		  d = get();
 		}
 		if(d == '\0' ) {
-		  //xerr('q', "TILE is a chunk of 8 characters."); 
-                  qerr();
+		  xerr('q', "TILE is a chunk of 8 characters."); 
 		}
 
 		/* .tile deals with chunks of 8 characters. We need to
@@ -574,8 +645,7 @@ struct mne *mp;
 		 */
 
 		if( i != 0 ) {
-		  //xerr('a', "Invalid character or terminated without 8 characters.");
-                  aerr();
+		  xerr('a', "Invalid character or terminated without 8 characters.");
 		  break;
 		}
 
@@ -588,16 +658,23 @@ struct mne *mp;
 		 */
 
 		if( c != d ) {
-		  //xerr('q', "Missing TILE terminator.");
-                  qerr();
+		  xerr('q', "Missing TILE terminator.");
 		  break;
 		}
 
 		break;
         default:
-                err('o');
+		opcycles = OPCY_ERR;
+                xerr('o', "Internal Opcode Error.");
                 break;
         }
+
+	if (opcycles == OPCY_NONE) {
+		opcycles = gbpg1[cb[0] & 0xFF];
+		if ((opcycles & OPCY_NONE) && (opcycles & OPCY_MASK)) {
+			opcycles = gbPage[opcycles & OPCY_MASK][cb[1] & 0xFF];
+		}
+	}
 }
 
 /*
