@@ -35,6 +35,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #endif
 #include <ctype.h>
 #include <errno.h>
+#include <unistd.h>
 #include "i_string.h"
 
 // prj
@@ -57,6 +58,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "cmdlexcl.h"
 
 bool jaj= false;
+int juj= 0;
 
 
 /*
@@ -66,9 +68,18 @@ bool jaj= false;
 
 cl_app::cl_app(void)
 {
+  save_std_attribs();
   sim= 0;
   in_files= new cl_ustrings(2, 2, "input files");
   options= new cl_options();
+  quiet= false;
+  if (app_start_at == 0)
+    app_start_at= dnow();
+  srnd(0);
+  set_console_mode();
+  period= 0;
+  cyc= 0;
+  acyc= 0;
 }
 
 cl_app::~cl_app(void)
@@ -76,6 +87,7 @@ cl_app::~cl_app(void)
   remove_simulator();
   delete commander;
   delete in_files;
+  delete ocon;
   delete options;
 }
 
@@ -84,116 +96,183 @@ cl_app::init(int argc, char *argv[])
 {
   sigpipe_off();
   cl_base::init();
-  set_name(cchars("application"));
+  if (!have_real_name())
+    set_name("application");
   mk_options();
   proc_arguments(argc, argv);
   class cl_cmdset *cmdset= new cl_cmdset();
   cmdset->init();
   build_cmdset(cmdset);
+  ocon= new cl_console_stdout(this);
+  ocon->init();
   commander= new cl_commander(this, cmdset/*, sim*/);
   commander->init();
+
   return(0);
 }
 
-/* Main cycle */
+void
+cl_app::read_conf_file(void)
+{
+  /* read config file (-C option) */
+  while (commander->config_console != NULL)
+    if (commander->input_avail())
+      commander->proc_input();
+}
 
-enum run_states {
-  rs_config,
-  rs_read_files,
-  rs_start,
-  rs_run
-};
+void
+cl_app::read_input_files(void)
+{
+  if (sim && (sim->uc != NULL))
+    {
+      int i;
+      for (i= 0; i < in_files->count; i++)
+	{
+	  const char *fname= (const char *)(in_files->at(i));
+	  long l;
+	  if ((l= sim->uc->read_file(fname, NULL)) >= 0)
+	    {
+	      sim->uc->reset();
+	    }
+	}
+    }
+}
+
+void
+cl_app::exec_startup_cmd(void)
+{
+  if (startup_command.nempty())
+    exec(startup_command);
+}
+
+int
+cl_app::check_start_options(void)
+{
+  class cl_option *o;
+
+  o= options->get_option("emulation");
+  bool emu_opt= false;
+  if (o)
+    o->get_value(&emu_opt);
+  if (sim && emu_opt)
+    {
+      sim->emulation(0);
+      return 0;
+    }
+  
+  o= options->get_option("go");
+  bool g_opt= false;
+  if (o)
+    o->get_value(&g_opt);
+  if (sim && g_opt)
+    sim->start(0, 0);
+
+  return 0;
+}
+
+/* Main cycle */
 
 int
 cl_app::run(void)
 {
   int done= 0;
-  double input_last_checked= 0;
-  class cl_option *o= options->get_option("go");
-  bool g_opt= false;
-  unsigned int cyc= 0;
-  enum run_states rs= rs_config;
-    
+
+  cperiod.set(cperiod_value());
+  read_conf_file();
+  read_input_files();
+  exec_startup_cmd();
+  check_start_options();
+  if (commander->consoles_prevent_quit() < 1)
+    done= 1;
   while (!done)
     {
-      if ((rs == rs_config) &&
-	  (commander->config_console == NULL))
-	{
-	  rs= rs_read_files;
-	}
-      if (rs == rs_read_files)
-	{
-	  if (sim && (sim->uc != NULL))
-	    {
-	      int i;
-	      for (i= 0; i < in_files->count; i++)
-		{
-		  char *fname= (char *)(in_files->at(i));
-		  long l;
-		  if ((l= sim->uc->read_file(fname, NULL)) >= 0)
-		    {
-		      ///*commander->all_printf*/printf("%ld words read from %s\n", l, fname);
-		    }
-		}
-	    }
-	  rs= rs_start;
-	}
-      if (rs == rs_start)
-	{
-	  //if (startup_command.nempty())
-	  //exec(startup_command);
-	  if (o)
-	    o->get_value(&g_opt);
-	  if (sim && g_opt)
-	    sim->start(0, 0);
-	  rs= rs_run;
-	}
-      ++cyc;
       if (!sim)
 	{
-	  commander->wait_input();
-	  done= commander->proc_input();
+	  if (commander->input_avail())
+	    done = commander->proc_input();
+	  loop_delay();
 	}
       else
-        {
-          if (sim->state & SIM_GO)
-            {
-	      if (cyc - input_last_checked > 10000)
-		{
-		  input_last_checked= cyc;
-		  if (sim->uc)
-		    sim->uc->touch();
-		  if (commander->input_avail())
-		    done= commander->proc_input();
-		}
-	      sim->step();
-	      if (jaj && commander->frozen_console)
-		{
-		  sim->uc->print_regs(commander->frozen_console),
-		    commander->frozen_console->dd_printf("\n");
-		}
-            }
-	  else
-	    {
-	      if (commander->input_avail())
-		done = commander->proc_input();
-              else
-                loop_delay();
-
-	      if (sim->uc)
-		sim->uc->touch();
-	    }
+	{
+	  acyc++;
 	  if (sim->state & SIM_QUIT)
 	    done= 1;
+	  else if (sim->state & SIM_GO)
+	    done= run_go();
+	  else if (sim->state & SIM_STARTEMU)
+	    {
+	      sim->state= SIM_EMU;
+	      sim->start_at= dnow();
+	      sim->uc->do_emu();
+	    }
+	  else if (sim->state & SIM_EMU)
+	    sim->uc->do_emu();
+	  else
+	    done= run_nogo();
 	}
-      commander->check();
+      //commander->check();
     }
+    
   return(0);
 }
 
+int
+cl_app::run_go(void)
+{
+  bool done= false;
+  if (++cyc > period)
+    {
+      cyc= 0;
+      if (sim->uc)
+	sim->uc->touch();
+      commander->check();
+      if (commander->input_avail())
+	done= commander->proc_input();
+    }
+  sim->step();
+  if (jaj)
+    {
+      class cl_console_base *c= commander->frozen_or_actual();
+      if (c)
+	sim->uc->print_regs(c),
+	  c->dd_printf("%d %lu\n", sim->uc->inst_ticks, sim->uc->ticks->get_ticks()),
+	  c->dd_printf("\n");
+    }
+  return done;
+}
+
+int
+cl_app::run_nogo(void)
+{
+  bool done= false;
+  commander->check();
+  if (commander->input_avail())
+    done = commander->proc_input();
+  else if (dnow() - app_start_at > 2.0)
+    loop_delay();
+  if (sim->uc)
+    sim->uc->touch();
+  return done;
+}
+
 void
 cl_app::done(void)
 {
+  restore_std_attribs();
+  bool bw= false;
+  class cl_option *o= application->options->get_option("black_and_white");
+  if (o)
+    {
+      o->get_value(&bw);
+      if (!bw)
+	{
+	  if (isatty(fileno(stdout)))
+	    {
+	      printf("\033[0m");
+	      fflush(0);
+	    }
+	}
+    }
 }
 
 
@@ -202,36 +281,43 @@ cl_app::done(void)
  */
 
 static void
-print_help(char *name)
+print_help(const char *name)
 {
   printf("%s: %s\n", name, VERSIONSTR);
-  printf("Usage: %s [-hHVvPgGwb] [-p prompt] [-t CPU] [-X freq[k|M]]\n"
+  printf("Usage: %s [-hHVvPgGEwlbBq] [-p prompt] [-t CPU] [-X freq[k|M]] [-R seed]\n"
 	 "       [-C cfg_file] [-c file] [-e command] [-s file] [-S optionlist]\n"
 	 "       [-I if_optionlist] [-o colorlist] [-a nr]\n"
 #ifdef SOCKET_AVAIL
-	 "       [-Z portnum] [-k portnum]"
+	 "       [-z portnum] [-Z portnum] [-k portnum]"
 #endif
 	 "\n"
 	 "       [files...]\n", name);
   printf
     (
+     /*
+      12345678901234567890123456789012345678901234567890123456789012345678901234567890
+               1         2         3         4         5         6         7         8
+      */
      "Options:\n"
      "  -t CPU       Type of CPU: 51, C52, 251, etc.\n"
      "  -X freq[k|M] XTAL frequency\n"
+     "  -R seed      Set the random number generator seed value\n"
      "  -C cfg_file  Read initial commands from `cfg_file' and execute them\n"
      "  -e command   Execute command on startup\n"
      "  -c file      Open command console on `file' (use `-' for std in/out)\n"
-     "  -Z portnum   Use localhost:portnum for command console\n"
-     "  -k portnum   Use localhost:portnum for serial I/O\n"
-     "  -s file      Connect serial interface uart0 to `file'\n"
+     "  -z portnum   Listen portnum for command console\n"
+     "  -Z portnum   Listen portnum for command console (no console on stdio)\n"
+     "  -k portnum   Listen portnum for serial I/O (obsolete, use -S)\n"
+     "  -s file      Connect serial interface uart0 to `file' (obsolete, use -S)\n"
      "  -S options   `options' is a comma separated list of options according to\n"
      "               serial interface. Know options are:\n"
      "                  uart=nr   number of uart (default=0)\n"
      "                  in=file   serial input will be read from file named `file'\n"
      "                  out=file  serial output will be written to `file'\n"
-     "                  port=nr   Use localhost:nr as server for serial line\n"
-     "                  iport=nr  Use localhost:nr as server for serial input\n"
-     "                  oport=nr  Use localhost:nr as server for serial output\n"
+     "                  port=nr   use localhost:nr as server for serial line\n"
+     "                  iport=nr  use localhost:nr as server for serial input\n"
+     "                  oport=nr  use localhost:nr as server for serial output\n"
+     "                  raw       perform non-interactive communication even on tty\n"
      "  -I options   `options' is a comma separated list of options according to\n"
      "               simulator interface. Known options are:\n"
      "                 if=memory[address]  turn on interface on given memory location\n"
@@ -239,15 +325,19 @@ print_help(char *name)
      "                 out=file            specify output file for IO\n"
      "  -p prompt    Specify string for prompt\n"
      "  -P           Prompt is a null ('\\0') character\n"
-     "  -o colors    List of color specification: what=colspec,...\n"
+     "  -o colors    `colors' is a list of color specification: what=colspec,...\n"
      "               where colspec is : separated list of color options\n"
      "               e.g.: prompt=b:white:black (bold white on black)\n"
-     "  -b           Black & white (non-color) console\n"
+     "  -l           Use light theme (default is dark)\n"
+     "  -b           Black & white (non-color) theme\n"
+     "  -B           Beep on breakpoints\n"
      "  -g           Go, start simulation\n"
      "  -G           Go, start simulation, quit on stop\n"
+     "  -E           Go, start simulation in emulation mode\n"
      "  -a nr        Specify size of variable space (default=256)\n"
      "  -w           Writable flash\n"
      "  -V           Verbose mode\n"
+     "  -q           Quiet mode (implies -b)\n"
      "  -v           Print out version number and quit\n"
      "  -H           Print out types of known CPUs and quit\n"
      "  -h           Print out this help and quit\n"
@@ -261,7 +351,9 @@ enum {
   SOPT_USART,
   SOPT_PORT,
   SOPT_IPORT,
-  SOPT_OPORT
+  SOPT_OPORT,
+  SOPT_RAW,
+  SOPT_ERROR
 };
 
 static const char *S_opts[]= {
@@ -272,6 +364,7 @@ static const char *S_opts[]= {
   /*[SOPT_PORT]=*/	"port",
   /*[SOPT_IPORT]=*/	"iport",
   /*[SOPT_OPORT]=*/	"oport",
+  /*[SOPT_RAW]=*/	"raw",
   NULL
 };
 
@@ -294,12 +387,12 @@ cl_app::proc_arguments(int argc, char *argv[])
   int i, c;
   char opts[100], *cp, *subopts, *value;
   char *cpu_type= NULL;
-  bool /*s_done= DD_FALSE,*/ k_done= false;
+  bool /*s_done= false,*/ k_done= false;
   //bool S_i_done= false, S_o_done= false;
 
-  strcpy(opts, "c:C:e:p:PX:vVt:s:S:I:a:whHgGJo:b_");
+  strcpy(opts, "qc:C:e:p:PX:vVt:s:S:I:a:whHgGEJo:blBR:_");
 #ifdef SOCKET_AVAIL
-  strcat(opts, "Z:r:k:");
+  strcat(opts, "Z:r:k:z:");
 #endif
 
   for (i= 0; i < argc; i++)
@@ -309,12 +402,22 @@ cl_app::proc_arguments(int argc, char *argv[])
 	  (strcmp(argv[i], "-args") == 0) ||
 	  (strcmp(argv[i], "-nx") == 0))
 	strcpy(argv[i], "-_");
+      if ((strcmp(argv[i], "-help")==0) ||
+	  (strcmp(argv[i], "--help")==0))
+	{
+	  print_help(get_name());
+	  exit(0);
+	}
     }
   
   while((c= getopt(argc, argv, opts)) != -1)
     switch (c)
       {
       case '_': break;
+      case 'q':
+	quiet= true;
+	options->set_value("black_and_white", this, bool(true));
+	break;
       case 'J': jaj= true; break;
       case 'g':
 	if (!options->set_value("go", this, true))
@@ -329,6 +432,11 @@ cl_app::proc_arguments(int argc, char *argv[])
 	  fprintf(stderr, "Warning: No \"quit\" option found "
 		  "to set by -G\n");
 	break;
+      case 'E':
+	if (!options->set_value("emulation", this, true))
+	  fprintf(stderr, "Warning: No \"emulation\" option found "
+		  "to set by -E\n");
+	break;
       case 'c':
 	if (!options->set_value("console_on", this, optarg))
 	  fprintf(stderr, "Warning: No \"console_on\" option found "
@@ -341,21 +449,36 @@ cl_app::proc_arguments(int argc, char *argv[])
 	break;
       case 'e':
 	startup_command+= optarg;
-	startup_command+= chars("\n");
+	startup_command+= chars(";"/*"\n"*/);
 	break;
+      case 'R':
+        srnd(atoi(optarg));
+        break;
 #ifdef SOCKET_AVAIL
+      case 'z':
+	{
+	  class cl_option *o;
+	  options->new_option(o= new cl_number_option(this, "default_port",
+						      "Default port to listen on (-z)"));
+	  o->init();
+	  o->hide();
+	  if (!options->set_value("default_port", this, strtol(optarg, NULL, 0)))
+	    fprintf(stderr, "Warning: No \"default_port\" option found"
+		    " to set parameter of -z as port number to listen on\n");
+	  break;
+	}	
       case 'Z': case 'r':
 	{
 	  // By Sandeep
 	  // Modified by DD
 	  class cl_option *o;
-	  options->new_option(o= new cl_number_option(this, cchars("port_number"),
+	  options->new_option(o= new cl_number_option(this, "port_number",
 						      "Listen on port (-Z)"));
 	  o->init();
 	  o->hide();
 	  if (!options->set_value("port_number", this, strtol(optarg, NULL, 0)))
 	    fprintf(stderr, "Warning: No \"port_number\" option found"
-		    " to set parameter of -Z as pot number to listen on\n");
+		    " to set parameter of -Z as port number to listen on\n");
 	  break;
 	}
 #endif
@@ -387,7 +510,7 @@ cl_app::proc_arguments(int argc, char *argv[])
 	    XTAL*= 1e6;
 	  if (XTAL == 0)
 	    {
-	      fprintf(stderr, "Xtal frequency must be greather than 0\n");
+	      fprintf(stderr, "Xtal frequency must be greater than 0\n");
 	      exit(1);
 	    }
 	  if (!options->set_value("xtal", this, XTAL))
@@ -461,38 +584,52 @@ cl_app::proc_arguments(int argc, char *argv[])
 	  if (!options->set_value("serial0_in_file", this, /*(void*)Ser_in*/s))
 	    fprintf(stderr, "Warning: No \"serial0_in_file\" option found to "
 		    "set parameter of -s as serial input file\n");
+	  free(s);
 	  s= format_string("\0010x%llx", (unsigned long long int)(fout));
 	  if (!options->set_value("serial0_out_file", this, /*Ser_out*/s))
 	    fprintf(stderr, "Warning: No \"serial0_out_file\" option found "
 		    "to set parameter of -s as serial output file\n");
+	  free(s);
 	  break;
 	}
 #endif
       case 'S':
 	{
 	  char *iname= NULL, *oname= NULL;
-	  int uart=0, port=0, iport= 0, oport= 0;
+	  int uart=0, port=0, iport= 0, oport= 0, so;
+	  bool ifirst= false, raw= false;
 	  subopts= optarg;
 	  while (*subopts != '\0')
 	    {
-	      switch (get_sub_opt(&subopts, S_opts, &value))
+	      so= get_sub_opt(&subopts, S_opts, &value);
+	      if (so != SOPT_RAW)
 		{
+		  if ((value == NULL) ||
+		      (*value == 0))
+		    so= SOPT_ERROR;
+		}
+	      switch (so)
+		{
+		case SOPT_ERROR:
+		  fprintf(stderr, "No value for -S suboption\n");
+		  exit(1);
+		  break;
+		case SOPT_RAW:
+		  raw= 1;
+		  break;
 		case SOPT_IN:
-		  if (value == NULL) {
-		    fprintf(stderr, "No value for -S in\n");
-		    exit(1);
-		  }
 		  iname= value;
+		  if (oname == NULL)
+		    ifirst= true;
 		  break;
 		case SOPT_OUT:
-		  if (value == NULL) {
-		    fprintf(stderr, "No value for -S out\n");
-		    exit(1);
-		  }
 		  oname= value;
 		  break;
 		case SOPT_UART: case SOPT_USART:
 		  uart= strtol(value, 0, 0);
+		  ifirst= false;
+		  iname= oname= NULL;
+		  port= iport= oport= 0;
 		  break;
 		case SOPT_PORT:
 		  port= strtol(value, 0, 0);
@@ -518,6 +655,30 @@ cl_app::proc_arguments(int argc, char *argv[])
 	    {
 	      char *s, *h;
 	      class cl_option *o;
+	      s= format_string("serial%d_raw", uart);
+	      if ((o= options->get_option(s)) == NULL)
+		{
+		  h= format_string("Use raw communication on uart%d files", uart);
+		  o= new cl_bool_option(this, s, h);
+		  o->init();
+		  o->hide();
+		  options->add(o);
+		  free(h);
+		}
+	      options->set_value(s, this, raw);
+	      free(s);
+	      s= format_string("serial%d_ifirst", uart);
+	      if ((o= options->get_option(s)) == NULL)
+		{
+		  h= format_string("Open input file for uart%d first", uart);
+		  o= new cl_bool_option(this, s, h);
+		  o->init();
+		  o->hide();
+		  options->add(o);
+		  free(h);
+		}
+	      options->set_value(s, this, ifirst);
+	      free(s);
 	      if (iname)
 		{
 		  s= format_string("serial%d_in_file", uart);
@@ -653,24 +814,50 @@ cl_app::proc_arguments(int argc, char *argv[])
 	}
       case 'o':
 	{
-	  chars s= optarg;
+	  const chars s= optarg;
 	  chars opt= s.token(",");
 	  while (opt.nempty())
 	    {
-	      printf("colspecopt=\"%s\"\n", (char*)opt);
-	      chars col_name, col_value;
+	      chars col_name, col_value, opt_name;
 	      col_name= opt.token("=");
 	      col_value=opt.token("=");
-	      printf("name=\"%s\" value=\"%s\"\n", (char*)col_name,(char*)col_value);
-	      class cl_option *o= options->get_option((char*)chars("","color_%s",(char*)col_name));
+	      opt_name.format("color_%s", col_name.c_str());
+	      class cl_option *o= options->get_option(opt_name);
 	      if (o)
 		o->set_value(col_value);
 	      opt= s.token(",");
 	    }
 	  break;
 	}
+      case 'l':
+	set_option_s("color_prompt", "green:bwhite");
+	set_option_s("color_prompt_console", "blue:bwhite");
+	set_option_s("color_command", "blue:bwhite");
+	set_option_s("color_answer", "black:bwhite");
+	set_option_s("color_result", "bblue:bwhite");
+	set_option_s("color_dump_address", "blue:bwhite");
+	set_option_s("color_dump_number", "bblack:bwhite");
+	set_option_s("color_dump_char", "black:bwhite");
+	set_option_s("color_comment", "magenta:bwhite");
+	set_option_s("color_error", "red:bwhite");
+	set_option_s("color_ui_mkey", "green:bwhite");
+	set_option_s("color_ui_mitem", "bblack:bwhite");
+	set_option_s("color_ui_label", "black:bwhite");
+	set_option_s("color_ui_time", "blue:bwhite");
+	set_option_s("color_ui_title", "magenta:bwhite");
+	set_option_s("color_ui_run", "black:green");
+	set_option_s("color_ui_stop", "white:red");
+	set_option_s("color_ui_bit0", "white:black");
+	set_option_s("color_ui_bit1", "bred:black");
+	set_option_s("color_debug", "magenta:bwhite");
+	break;
+      case 'B':
+	if (!options->set_value("beep_break", this, (bool)true))
+	  fprintf(stderr, "Warning: No \"debug\" option found to set "
+		  "by -B parameter\n");	
+	break;
       case 'h':
-	print_help(cchars("s51"));
+	print_help(get_name());
 	exit(0);
 	break;
       case 'H':
@@ -728,11 +915,11 @@ cl_app::get_cmd(class cl_cmdline *cmdline)
   return(0);
 }
 */
-long
+t_mem
 cl_app::eval(chars expr)
 {
   expr_result= 0;
-  uc_yy_set_string_to_parse((char*)expr);
+  uc_yy_set_string_to_parse(expr);
   yyparse();
   uc_yy_free_string_to_parse();
   return expr_result;
@@ -741,15 +928,11 @@ cl_app::eval(chars expr)
 void
 cl_app::exec(chars line)
 {
-  class cl_console_base *c= commander->frozen_console;
-  if (c == NULL)
-    {
-      c= new cl_console_dummy();
-      c->init();
-    }
+  class cl_console_base *c= commander->frozen_or_actual();
   do
     {
-      class cl_cmdline *cmdline= new cl_cmdline(this, (char*)line, c);
+      c->un_redirect();
+      class cl_cmdline *cmdline= new cl_cmdline(this, line, c);
       cmdline->init();
       class cl_cmd *cm= commander->cmdset->get_cmd(cmdline, false/*c->is_interactive()*/);
       if (cm)
@@ -761,16 +944,47 @@ cl_app::exec(chars line)
 	  char *e= cmdline->cmd;
 	  if (strlen(e) > 0)
 	    {
-	      long l= eval(e);
-	      c->dd_printf("%ld\n", l);
+	      t_mem l= eval(e);
+	      c->dd_color("result");
+	      c->print_expr_result(l, NULL);
+	    }
+	}
+      line= cmdline->rest;
+      delete cmdline;
+    }
+  while (!line.empty());
+}
+
+void
+cl_app::exec(chars line, class cl_console_base *con)
+{
+  if (!con)
+    return;
+  do
+    {
+      con->un_redirect();
+      class cl_cmdline *cmdline= new cl_cmdline(this, line, con);
+      cmdline->init();
+      class cl_cmd *cm= commander->cmdset->get_cmd(cmdline, false/*c->is_interactive()*/);
+      if (cm)
+	{
+	  cm->work(this, cmdline, con);
+	}
+      else if (cmdline->get_name() != 0)
+	{
+	  char *e= cmdline->cmd;
+	  if (strlen(e) > 0)
+	    {
+	      t_mem l= eval(e);
+	      con->dd_color("result");
+	      con->print_expr_result(l, NULL);
+	      
 	    }
 	}
       line= cmdline->rest;
       delete cmdline;
     }
   while (!line.empty());
-  if (c != commander->frozen_console)
-    delete c;
 }
 
 /*
@@ -815,6 +1029,10 @@ cl_app::build_cmdset(class cl_cmdset *cmdset)
   cmdset->add(cmd= new cl_super_cmd("conf", 0, cset));
   cmd->init();
   set_conf_help(cmd);
+
+  cmd= new cl_ver_cmd("version", 0);
+  cmdset->add(cmd);
+  cmd->init();
   
   cmd= new cl_help_cmd("help", 0);
   cmdset->add(cmd);
@@ -823,13 +1041,21 @@ cl_app::build_cmdset(class cl_cmdset *cmdset)
 
   cmdset->add(cmd= new cl_quit_cmd("quit", 0));
   cmd->init();
-
+  cmd->add_name("exit");
+  
   cmdset->add(cmd= new cl_kill_cmd("kill", 0));
   cmd->init();
 
   cmdset->add(cmd= new cl_exec_cmd("exec", 0));
   cmd->init();
 
+  cmdset->add(cmd= new cl_echo_cmd("echo", 0));
+  cmd->init();
+  cmd->add_name("print");
+
+  cmdset->add(cmd= new cl_dev_cmd("dev", 0));
+  cmd->init();
+
   cmdset->add(cmd= new cl_expression_cmd("expression", 0));
   cmd->init();
   cmd->add_name("let");
@@ -956,7 +1182,7 @@ cl_app::mk_options(void)
   options->new_option(o= new cl_float_option(this, "xtal",
 					     "Frequency of XTAL in Hz"));
   o->init();
-  o->set_value(11059200.0);
+  o->set_value(0.0);
 
   options->new_option(o= new cl_string_option(this, "cpu_type",
 					      "Type of controller (-t)"));
@@ -973,6 +1199,11 @@ cl_app::mk_options(void)
   o->init();
   o->hide();
   
+  options->new_option(o= new cl_bool_option(this, "emulation",
+					    "Emulate on start (-E)"));
+  o->init();
+  o->hide();
+  
   options->new_option(o= new cl_number_option(this, "var_size",
 					      "Size of variable space (-a)"));
   o->init();
@@ -1000,91 +1231,125 @@ cl_app::mk_options(void)
 					    "Print breakpoint script before execute"));
   o->init();
 
+  options->new_option(o= new cl_bool_option(this, "beep_break",
+					    "Beep at breakpoint hit (-B)"));
+  o->init();
+  o->set_value((bool)false);
+
+  options->new_option(o= new cl_string_option(this, "expression_format",
+					      "Format specifier to print expression result"));
+  o->init();
+  o->set_value("d");
+  
   options->new_option(o= new cl_string_option(this, "color_prompt",
 					      "Prompt color"));
   o->init();
-  o->set_value((char*)"bwhite:black");
+  o->set_value("bwhite:black");
   
   options->new_option(o= new cl_string_option(this, "color_prompt_console",
 					      "Color of console number in prompt"));
   o->init();
-  o->set_value((char*)"yellow:black");
+  o->set_value("yellow:black");
   
   options->new_option(o= new cl_string_option(this, "color_command",
 					      "Color of entered command"));
   o->init();
-  o->set_value((char*)"green:black");
+  o->set_value("green:black");
   
   options->new_option(o= new cl_string_option(this, "color_answer",
 					      "Answer color"));
   o->init();
-  o->set_value((char*)"bwhite:black");
+  o->set_value("bwhite:black");
   
   options->new_option(o= new cl_string_option(this, "color_result",
 					      "Result of expression"));
   o->init();
-  o->set_value((char*)"byellow:black");
+  o->set_value("byellow:black");
   
   options->new_option(o= new cl_string_option(this, "color_dump_address",
 					      "Address color in dump"));
   o->init();
-  o->set_value((char*)"byellow:black");
+  o->set_value("byellow:black");
   
+  options->new_option(o= new cl_string_option(this, "color_dump_label",
+					      "Label color in dump"));
+  o->init();
+  o->set_value((char*)"bgreen:black");
+
   options->new_option(o= new cl_string_option(this, "color_dump_number",
 					      "Value color in dump"));
   o->init();
-  o->set_value((char*)"white:black");
+  o->set_value("white:black");
   
   options->new_option(o= new cl_string_option(this, "color_dump_char",
 					      "Text color in dump"));
   o->init();
-  o->set_value((char*)"green:black");
+  o->set_value("green:black");
+
+  options->new_option(o= new cl_string_option(this, "color_comment",
+					      "Comment color in disassembly"));
+  o->init();
+  o->set_value("magenta:black");
   
   options->new_option(o= new cl_string_option(this, "color_error",
 					      "Text color in error messages"));
   o->init();
-  o->set_value((char*)"red:black");
+  o->set_value("red:black");
   
   options->new_option(o= new cl_string_option(this, "color_debug",
 					      "Color of debug messages"));
   o->init();
-  o->set_value((char*)"magenta:black");
+  o->set_value("magenta:black");
   
   options->new_option(o= new cl_string_option(this, "color_ui_mkey",
 					      "Menu-key color on UI display"));
   o->init();
-  o->set_value((char*)"b:yellow:black");
+  o->set_value("b:yellow:black");
   
   options->new_option(o= new cl_string_option(this, "color_ui_mitem",
 					      "Menu-item color on UI display"));
   o->init();
-  o->set_value((char*)"bwhite:black");
+  o->set_value("bwhite:black");
   
   options->new_option(o= new cl_string_option(this, "color_ui_label",
 					      "Label color on UI display"));
   o->init();
-  o->set_value((char*)"white:black");
+  o->set_value("white:black");
   
   options->new_option(o= new cl_string_option(this, "color_ui_time",
 					      "Color of time-value on UI display"));
   o->init();
-  o->set_value((char*)"bblue:black");
+  o->set_value("bblue:black");
   
   options->new_option(o= new cl_string_option(this, "color_ui_title",
 					      "Title color on UI display"));
   o->init();
-  o->set_value((char*)"bmagenta:black");
+  o->set_value("bmagenta:black");
   
   options->new_option(o= new cl_string_option(this, "color_ui_run",
 					      "Run state color on UI display"));
   o->init();
-  o->set_value((char*)"black:green");
+  o->set_value("black:green");
+  
+  options->new_option(o= new cl_string_option(this, "color_ui_bit0",
+					      "Bit 0 color on UI display"));
+  o->init();
+  o->set_value("white:black");
+  
+  options->new_option(o= new cl_string_option(this, "color_ui_bit1",
+					      "Bit 1 color on UI display"));
+  o->init();
+  o->set_value("bred:black");
   
   options->new_option(o= new cl_string_option(this, "color_ui_stop",
 					      "Stop state color on UI display"));
   o->init();
-  o->set_value((char*)"white:red");
+  o->set_value("white:red");
   
+  options->new_option(o= new cl_number_option(this, "label_width",
+					      "Space to allow for labels in dumps and disassembly (-1 for auto)"));
+  o->init();
+  o->set_value((long)-1);
 }
 
 
@@ -1131,4 +1396,14 @@ cl_app::debug(const char *format, ...)
 }
 
 
+void
+cl_app::set_option_s(const char *opt_name, const char *new_value)
+{
+  class cl_option *o= options->get_option(opt_name);
+  if (o)
+    {
+      o->set_value(new_value);
+    }
+}
+
 /* End of app.cc */
