@@ -25,19 +25,21 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 02111-1307, USA. */
 /*@1@*/
 
-#include "ddconfig.h"
+//#include "ddconfig.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
-#include "i_string.h"
+#include <string.h>
+//#include "i_string.h"
 
 // prj
-#include "pobjcl.h"
+//#include "pobjcl.h"
 
 // sim
 #include "simcl.h"
-#include "memcl.h"
+//#include "memcl.h"
+#include "dregcl.h"
 
 // local
 #include "portcl.h"
@@ -132,21 +134,15 @@ cl_avr::init(void)
   for (i= 0; sfr_tabl[i].name != NULL; i++)
     {
       if (type->type & sfr_tabl[i].cpu_type)
-	{
-	  class cl_var *v;
-	  vars->add(v= new cl_var(chars(sfr_tabl[i].name),
-				  ram,
-				  sfr_tabl[i].addr, ""));
-	  v->init();
-	}
+        vars->add(sfr_tabl[i].name, ram, sfr_tabl[i].addr, 7, 0, "");
     }
   return(0);
 }
 
-char *
+const char *
 cl_avr::id_string(void)
 {
-  return((char*)"unspecified AVR");
+  return("unspecified AVR");
 }
 
 
@@ -159,6 +155,10 @@ cl_avr::mk_hw_elements(void)
 {
   class cl_hw *h;
   cl_uc::mk_hw_elements();
+
+  add_hw(h= new cl_dreg(this, 0, "dreg"));
+  h->init();
+
   add_hw(h= new cl_port(this));
   h->init();
 }
@@ -179,7 +179,7 @@ cl_avr::make_memories(void)
   class cl_address_decoder *ad;
   class cl_memory_chip *chip;
 
-  chip= new cl_memory_chip("rom_chip", 0x10000, 16);
+  chip= new cl_chip16("rom_chip", 0x10000, 16);
   chip->init();
   memchips->add(chip);
   ad= new cl_address_decoder(as= rom/*address_space(MEM_ROM_ID)*/,
@@ -188,7 +188,7 @@ cl_avr::make_memories(void)
   as->decoders->add(ad);
   ad->activate(0);
 
-  chip= new cl_memory_chip("iram_chip", 0x80, 8);
+  chip= new cl_chip8("iram_chip", 0x80, 8);
   chip->init();
   memchips->add(chip);
   ad= new cl_address_decoder(as= ram/*address_space(MEM_IRAM_ID)*/,
@@ -210,142 +210,141 @@ cl_avr::dis_tbl(void)
 }
 
 char *
-cl_avr::disass(t_addr addr, const char *sep)
+cl_avr::disass(t_addr addr)
 {
-  char work[256], temp[20];
-  char *buf, *p, *t, *s;
+  chars work, temp;
   const char *b;
   uint code, data= 0;
+  t_addr operand;
   int i;
+  bool first;
+  
+  work= "";
 
-  p= work;
-
-  code= rom/*get_mem*/->get(/*MEM_ROM_ID,*/ addr);
+  code= rom->get(addr);
   i= 0;
   while ((code & dis_tbl()[i].mask) != dis_tbl()[i].code &&
 	 dis_tbl()[i].mnemonic)
     i++;
   if (dis_tbl()[i].mnemonic == NULL)
     {
-      buf= (char*)malloc(30);
-      strcpy(buf, "UNKNOWN/INVALID");
-      return(buf);
+      return(strdup("UNKNOWN/INVALID"));
     }
   b= dis_tbl()[i].mnemonic;
 
+  first= true;
   while (*b)
     {
+      if ((*b == ' ') && first)
+	{
+	  first= false;
+	  while (work.len() < 6) work.append(' ');
+	}
       if (*b == '%')
 	{
 	  b++;
+	  temp= "";
 	  switch (*(b++))
 	    {
 	    case 'd': // Rd   .... ...d dddd ....  0<=d<=31
-	      if (!/*get*/addr_name(data= (code&0x01f0)>>4, /*sfr_tbl()*/ram, temp))
-		sprintf(temp, "r%d", data);
+	      temp.format("r%d", data);
+	      addr_name(data= (code&0x01f0)>>4, ram, &temp);
 	      break;
 	    case 'D': // Rd   .... .... dddd ....  16<=d<=31
-	      if (!/*get*/addr_name(data= 16+((code&0xf0)>>4), /*sfr_tbl()*/ram, temp))
-		sprintf(temp, "r%d", data);
+	      temp.format("r%d", data);
+	     addr_name(data= 16+((code&0xf0)>>4), ram, &temp);
 	      break;
 	    case 'K': // K    .... KKKK .... KKKK  0<=K<=255
-	      sprintf(temp, "%d", ((code&0xf00)>>4)|(code&0xf));
+	      temp.appendf("%d", ((code&0xf00)>>4)|(code&0xf));
 	      break;
 	    case 'r': // Rr   .... ..r. .... rrrr  0<=r<=31
-	      if (!/*get*/addr_name(data= ((code&0x0200)>>5)|(code&0x000f),
-				    /*sfr_tbl()*/ram, temp))
-		sprintf(temp, "r%d", data);
+	      temp.format("r%d", data);
+	      addr_name(data= ((code&0x0200)>>5)|(code&0x000f),
+			ram, &temp);
 	      break;
 	    case '2': // Rdl  .... .... ..dd ....  dl= {24,26,28,30}
-	      if (!/*get*/addr_name(data= 24+(2*((code&0x0030)>>4)),
-				    /*sfr_tbl()*/ram, temp))
-		sprintf(temp, "r%d", data);
+	      temp.format("r%d", data);
+	      addr_name(data= 24+(2*((code&0x0030)>>4)),
+			ram, &temp);
 	      break;
 	    case '6': // K    .... .... KK.. KKKK  0<=K<=63
-	      sprintf(temp, "%d", ((code&0xc0)>>2)|(code&0xf));
+	      temp.format("%d", ((code&0xc0)>>2)|(code&0xf));
 	      break;
 	    case 's': // s    .... .... .sss ....  0<=s<=7
-	      sprintf(temp, "%d", (code&0x70)>>4);
+	      temp.format("%d", (code&0x70)>>4);
 	      break;
 	    case 'b': // b    .... .... .... .bbb  0<=b<=7
-	      sprintf(temp, "%d", code&0x7);
+	      temp.format("%d", code&0x7);
 	      break;
 	    case 'k': // k    .... ..kk kkkk k...  -64<=k<=+63
 	      {
 		int k= (code&0x3f8)>>3;
 		if (code&0x200)
 		  k|= -128;
-		sprintf(temp, "0x%06x", k+1+(signed int)addr);
+		k+= 1 + (signed int)addr;
+		temp.format("0x%06x", k);
+		addr_name(k, rom, &temp);
 		break;
 	      }
 	    case 'A': // k    .... ...k kkkk ...k  0<=k<=64K
 	              //      kkkk kkkk kkkk kkkk  0<=k<=4M
-	      sprintf(temp, "0x%06x",
-		      (((code&0x1f0)>>3)|(code&1))*0x10000+
-		      (uint)rom->get/*_mem*/(/*MEM_ROM_ID,*/ addr+1));
+	      operand= (((code&0x1f0)>>3)|(code&1))*0x10000+
+		       (uint)rom->get(addr+1);
+	      temp.format("0x%06x", operand);
+	      addr_name(operand, rom, &temp);
 	      break;
 	    case 'P': // P    .... .... pppp p...  0<=P<=31
 	      data= (code&0xf8)>>3;
-	      if (!/*get*/addr_name(data+0x20, /*sfr_tbl()*/ram, temp))
-		sprintf(temp, "%d", data);
+	      temp.format("%d", data);
+	      addr_name(data+0x20, ram, &temp);
 	      break;
 	    case 'p': // P    .... .PP. .... PPPP  0<=P<=63
 	      data= ((code&0x600)>>5)|(code&0xf);
-	      if (!/*get*/addr_name(data+0x20, /*sfr_tbl()*/ram, temp))
-		sprintf(temp, "%d", data);
+	      temp.format("%d", data);
+	      addr_name(data+0x20, ram, &temp);
 	      break;
 	    case 'q': // q    ..q. qq.. .... .qqq  0<=q<=63
-	      sprintf(temp, "%d",
-		      ((code&0x2000)>>8)|((code&0xc00)>>7)|(code&7));
+	      temp.format("%d",
+			  ((code&0x2000)>>8)|((code&0xc00)>>7)|(code&7));
 	      break;
 	    case 'R': // k    SRAM address on second word 0<=k<=65535
-	      sprintf(temp, "0x%06x", (uint)rom->get/*_mem*/(/*MEM_ROM_ID,*/ addr+1));
+	      operand= rom->get(addr+1);
+	      temp.format("0x%06x", operand);
+	      addr_name(operand, ram, &temp);
 	      break;
 	    case 'a': // k    .... kkkk kkkk kkkk  -2k<=k<=2k
 	      {
 		int k= code&0xfff;
 		if (code&0x800)
 		  k|= -4096;
-		sprintf(temp, "0x%06x",
-			(int)rom->validate_address(k+1+(signed int)addr));
+		k= rom->validate_address(k+1+(signed int)addr);
+		temp.format("0x%06x", k);
+		addr_name(k, rom, &temp);
 		break;
 	      }
 	    default:
-	      strcpy(temp, "?");
+	      temp= "?";
 	      break;
 	    }
-	  t= temp;
-	  while (*t)
-	    *(p++)= *(t++);
+	  work+= temp;
 	}
       else
-	*(p++)= *(b++);
+	work+= *(b++);
     }
-  *p= '\0';
 
-  p= strchr(work, ' ');
-  if (!p)
-    {
-      buf= strdup(work);
-      return(buf);
-    }
-  if (sep == NULL)
-    buf= (char *)malloc(6+strlen(p)+1);
-  else
-    buf= (char *)malloc((p-work)+strlen(sep)+strlen(p)+1);
-  for (p= work, s= buf; *p != ' '; p++, s++)
-    *s= *p;
-  p++;
-  *s= '\0';
-  if (sep == NULL)
-    {
-      while (strlen(buf) < 6)
-	strcat(buf, " ");
-    }
-  else
-    strcat(buf, sep);
-  strcat(buf, p);
-  return(buf);
+  return(strdup(work.c_str()));
+}
+
+void
+cl_avr::analyze_start(void)
+{
+  return;
+}
+
+void
+cl_avr::analyze(t_addr addr)
+{
+  return;
 }
 
 
@@ -355,7 +354,8 @@ cl_avr::print_regs(class cl_console_base *con)
   uchar data, sreg= ram->get(SREG);
   uint x, y, z;
 
-  ram->dump(0, 31, 16, con/*->get_fout()*/);
+  class cl_dump_ads ads(0, 31);
+  ram->dump(0, /*0, 31*/&ads, 16, con);
 
   con->dd_color("answer");
   con->dd_printf("ITHSVNZC  SREG= 0x%02x %3d %c\n",
@@ -666,16 +666,8 @@ cl_avr::exec_inst(void)
 	break;
       }
     }
-  /*if (PC)
-    PC--;
-  else
-  PC= get_mem_size(MEM_ROM_ID)-1;*/
   class cl_error_unknown_code *e= new cl_error_unknown_code(this);
   error(e);
-  return(resGO);
-  PC= rom->inc_address(PC, -1);
-  //tick(-clock_per_cycle());
-  sim->stop(resINV_INST);
   return(resINV_INST);
 }
 
