@@ -30,6 +30,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include <stdlib.h>
 
 //#include "ddconfig.h"
+#include "globals.h"
 
 // sim
 //#include "brkcl.h"
@@ -66,61 +67,85 @@ COMMAND_DO_WORK_UC(cl_break_cmd)
   else if (cmdline->syntax_match(uc, CELL NUMBER STRING STRING)) {
     hit= params[1]->value.number;
     mem= uc->address_space(params[0]->value.cell, &addr);
-    if (!mem)
-      return syntax_error(con),	false;
     s= params[2]->get_svalue();
     if (s && *s &&
 	(strcmp(s, "if") == 0))
       cond= params[3]->get_svalue();
-    if (mem == uc->rom)
+    if (mem && (mem == uc->rom))
       do_fetch(uc, addr, hit, cond, con);
     else
       {
-	do_event(uc, mem, 'r', addr, hit, cond, con);
-	do_event(uc, mem, 'w', addr, hit, cond, con);
+	if (!mem)
+	  {
+	    do_event(uc, params[0]->value.cell, 'r', hit, cond, con);
+	    do_event(uc, params[0]->value.cell, 'w', hit, cond, con);
+	  }
+	else
+	  {
+	    do_event(uc, mem, 'r', addr, hit, cond, con);
+	    do_event(uc, mem, 'w', addr, hit, cond, con);
+	  }
       }
   }
   else if (cmdline->syntax_match(uc, CELL STRING STRING)) {
     hit= 1;
     mem= uc->address_space(params[0]->value.cell, &addr);
-    if (!mem)
-      return syntax_error(con),	false;
     s= params[1]->get_svalue();
     if (s && *s &&
 	(strcmp(s, "if") == 0))
       cond= params[2]->get_svalue();
-    if (mem == uc->rom)
+    if (mem && (mem == uc->rom))
       do_fetch(uc, addr, hit, cond, con);
     else
       {
-	do_event(uc, mem, 'r', addr, hit, cond, con);
-	do_event(uc, mem, 'w', addr, hit, cond, con);
+	if (!mem)
+	  {
+	    do_event(uc, params[0]->value.cell, 'r', hit, cond, con);
+	    do_event(uc, params[0]->value.cell, 'w', hit, cond, con);
+	  }
+	else
+	  {
+	    do_event(uc, mem, 'r', addr, hit, cond, con);
+	    do_event(uc, mem, 'w', addr, hit, cond, con);
+	  }
       }
   }
   else if (cmdline->syntax_match(uc, CELL NUMBER)) {
     hit= params[1]->value.number;
     mem= uc->address_space(params[0]->value.cell, &addr);
-    if (!mem)
-      return syntax_error(con),	false;
-    if (mem == uc->rom)
+    if (mem && (mem == uc->rom))
       do_fetch(uc, addr, hit, cond, con);
     else
       {
-	do_event(uc, mem, 'r', addr, hit, cond, con);
-	do_event(uc, mem, 'w', addr, hit, cond, con);
+	if (!mem)
+	  {
+	    do_event(uc, params[0]->value.cell, 'r', hit, cond, con);
+	    do_event(uc, params[0]->value.cell, 'w', hit, cond, con);
+	  }
+	else
+	  {
+	    do_event(uc, mem, 'r', addr, hit, cond, con);
+	    do_event(uc, mem, 'w', addr, hit, cond, con);
+	  }
       }
   }
   else if (cmdline->syntax_match(uc, CELL)) {
     hit= 1;
     mem= uc->address_space(params[0]->value.cell, &addr);
-    if (!mem)
-      return syntax_error(con),	false;
-    if (mem == uc->rom)
+    if (mem && (mem == uc->rom))
       do_fetch(uc, addr, hit, cond, con);
     else
       {
-	do_event(uc, mem, 'r', addr, hit, cond, con);
-	do_event(uc, mem, 'w', addr, hit, cond, con);
+	if (mem)
+	  {
+	    do_event(uc, mem, 'r', addr, hit, cond, con);
+	    do_event(uc, mem, 'w', addr, hit, cond, con);
+	  }
+	else
+	  {
+	    do_event(uc, params[0]->value.cell, 'r', hit, cond, con);
+	    do_event(uc, params[0]->value.cell, 'w', hit, cond, con);
+	  }
       }
   }
   else if (cmdline->syntax_match(uc, MEMORY STRING ADDRESS NUMBER STRING STRING)) {
@@ -194,7 +219,19 @@ COMMAND_DO_WORK_UC(cl_break_cmd)
 CMDHELP(cl_break_cmd,
 	"break addr [hit [if expr]] | break mem_type r|w addr [hit [if expr]]",
 	"Set fix or event breakpoint",
-	"long help of break")
+	"Fix breakpoint stops execution if instruction fetch happens at the\n"
+	"specified address. If the first parameter is an address then fix\n"
+	"breakpoint is created. Address can be a number or a name of a \n"
+	"variable. In this case variable must point to an address space,\n"
+	"otherwise fetch breakpoint is created.\n"
+	"Fetch breakpoint will be hit if read or write operation happens\n"
+	"at the specified address in the specified address space during\n"
+	"the instruction execution.\n"
+	"`Hit' parameter can be used to specify how many times the breakpoint\n"
+	"must be hit before acception.\n"
+	"`Expr' parameter can specify an expression which is evaluated\n"
+	"at every hit of the breakpint and it will be accepted only when\n"
+	"the result is true.")
 
 void
 cl_break_cmd::do_fetch(class cl_uc *uc,
@@ -217,9 +254,10 @@ cl_break_cmd::do_fetch(class cl_uc *uc,
       b->init();
       b->cond= cond;
       uc->fbrk->add_bp(b);
-      con->dd_printf("Breakpoint %d at 0x%06x: ", b->nr, AI(addr));
-      uc->print_disass(addr, con, false);
-      con->dd_printf(" (cond=\"%s\")\n", cond.c_str());
+      con->dd_printf("Breakpoint %d at 0x%06x (cond=\"%s\")\n",
+		     b->nr, AI(addr), cond.c_str());
+      uc->print_disass(addr, con);
+      //con->dd_printf(" (cond=\"%s\")\n", cond.c_str());
     }
 }
 
@@ -243,6 +281,26 @@ cl_break_cmd::do_event(class cl_uc *uc,
     con->dd_printf("Couldn't make event breakpoint\n");
 }
 
+void
+cl_break_cmd::do_event(class cl_uc *uc,
+		       class cl_memory_cell *cell,
+		       char op, int hit,
+		       chars cond,
+		       class cl_console_base *con)
+{
+  class cl_ev_brk *b= NULL;
+
+  b= uc->mk_ebrk(perm, cell, op, hit);
+  if (b)
+    {
+      b->init();
+      b->cond= cond;
+      uc->ebrk->add_bp(b);
+    }
+  else
+    con->dd_printf("Couldn't make event breakpoint\n");
+}
+
 
 /*
  * CLEAR address
@@ -287,7 +345,8 @@ COMMAND_DO_WORK_UC(cl_clear_cmd)
 CMDHELP(cl_clear_cmd,
 	"clear [addr...]",
 	"Clear fix breakpoint",
-	"long help of clear")
+	"Delete fix breakpoint(s) from the specified address(es).\n"
+	"To delete fetch breakpoints, `delete' command must be used.")
 
 /*
  * DELETE nr nr ...
@@ -323,10 +382,12 @@ COMMAND_DO_WORK_UC(cl_delete_cmd)
 CMDHELP(cl_delete_cmd,
 	"delete [nr...]",
 	"Delete breakpoint(s)",
-	"long help of clear")
+	"Delete fix or fetch breakpoint by its id number. `info break' command\n"
+	"can be used to list id number of the breakpoints.\n"
+	"If parameter is not used then all breakpoints will be deleted.")
 
 /*
- * COMMANDS [nr] cmdstring...
+ * COMMANDS nr [cmdstring...]
  */
 
 COMMAND_DO_WORK_UC(cl_commands_cmd)
@@ -376,6 +437,7 @@ COMMAND_DO_WORK_UC(cl_commands_cmd)
 
   if (!s.empty())
     {
+      s.rip("\"\n\t\v ");
       b->commands= s;
     }
   else
@@ -386,8 +448,89 @@ COMMAND_DO_WORK_UC(cl_commands_cmd)
 }
 
 CMDHELP(cl_commands_cmd,
-	"commands [breakpoint-nr]",
-	"command_string",
-	"long help of commands")
+	"commands breakpoint-nr [command_string]",
+	"Add/delete a command to/from a breakpoint",
+	"Breakpoint command is executed every time when the breakpoint is\n"
+	"accepted.\n"
+	"First parameter is id number of the breakpoint to be modify.\n"
+	"If command string is not used then command will be deleted from\n"
+	"the specified breakpoint.\n"
+	"If any string is used after the breakpoint number, the whole\n"
+	"string is copied into the breakpoint as the command.\n"
+	"If command separator (semicolon) is used, it will be part of the\n"
+	"breakpoint command.")
+
+
+/*
+ * DISPLAY [[/fmt] expr]
+ */
+
+COMMAND_DO_WORK_UC(cl_display_cmd)
+{
+  int i= 0;
+  chars fmt, exp;
+  const char *s;
+
+  for (i=0;i<cmdline->tokens->get_count();i++)
+    {
+      s= cmdline->tokens->at(i);
+      if (s && *s)
+	{
+	  if (s[0] == '/')
+	    fmt= s;
+	  else
+	    exp= s;
+	}
+    }
+  
+  if (exp.nempty())
+    {
+      // add new expr
+      class cl_display *d= new cl_display(fmt, exp);
+      uc->displays->add(d);
+    }
+  else
+    {
+      uc->displays->do_display(con);
+    }
+  return false;
+}
+
+CMDHELP(cl_display_cmd,
+	"display [[/fmt] expr]",
+	"Set expression to be displayed on breakpoint",
+	"/fmt can specify format as in `expr' command.\n"
+	"If no argument, print all expressions and their values.\n")
+
+
+/*
+ * UNDISPLAY [num]
+ */
+
+COMMAND_DO_WORK_UC(cl_undisplay_cmd)
+{
+  class cl_cmd_arg *params[1]= { cmdline->param(0) };
+
+  if (params[0]==NULL)
+    {
+      uc->displays->free_all();
+    }
+  else if (cmdline->syntax_match(uc, NUMBER))
+    {
+      uc->displays->undisplay(params[0]->value.number);
+    }
+  else
+    con->dd_printf("Syntax error.\n");
+  return false;
+}
+
+CMDHELP(cl_undisplay_cmd,
+	"undisplay [num]",
+	"Remove some expression to be displayed at breakpoint stop",
+	"Argument is number of expression to be removed,\n"
+	"\"display\" command can be used without argument to\n"
+	"check id number of the expressions.\n"
+	"No argument means remove all expressions.\n")
+
 
 /* End of cmd.src/cmd_bp.cc */
