@@ -79,7 +79,7 @@ COMMAND_DO_WORK_SIM(cl_run_cmd)
       if (!sim->uc->inst_at(start))
 	con->dd_printf("Warning: maybe not instruction at 0x%06x\n",
 		       AI(start));
-      sim->uc->PC= start;
+      sim->uc->set_PC(start);
       if (params[1])
 	{
 	  if (start == end)
@@ -100,18 +100,19 @@ COMMAND_DO_WORK_SIM(cl_run_cmd)
 	}
     }
   con->dd_printf("Simulation started, PC=0x%06x\n", AI(sim->uc->PC));
-  /*
-  if (sim->uc->fbrk_at(sim->uc->PC))
-    sim->uc->do_inst(1);
-  */
   sim->start(con, 0);
   return(false);
 }
 
 CMDHELP(cl_run_cmd,
 	"run [start [stop]]",
-	"Go",
-	"long help of run")
+	"Start execution in foreground",
+	"Start simulation at given address (first parameter) or at\n"
+	"actual PC value. If second parameter is used, a breakpoint\n"
+	"will be placed at that address which stops the simulation\n"
+	"when reached. The run command stops the actual console which\n"
+	"became frozen. Pressing ENTER key on this console stops the\n"
+	"simulation.\n")
 
 /*
  * Command: stop
@@ -130,8 +131,10 @@ COMMAND_DO_WORK_SIM(cl_stop_cmd)
 
 CMDHELP(cl_stop_cmd,
 	"stop",
-	"Stop",
-	"long help of stop")
+	"Stop simulation.",
+	"This command simply stops the simulation.\n"
+	"After stop, value of actual PC, code memory and the\n"
+	"disassembled instruction is printed.\n")
 
 /*
  * Command: step
@@ -235,9 +238,19 @@ COMMAND_DO_WORK_SIM(cl_step_cmd)
 }
 
 CMDHELP(cl_step_cmd,
-	"step [number[unit]]",
-	"Step",
-	"long help of step")
+	"step [number [unit]]",
+	"Execute one or specified instructions.",
+	"Step executes one instruction or number of specified instructions.\n"
+	"Number of steps can be specified in different units. Known units:\n"
+	" s,sec       Simulated execution time in seconds\n"
+	" ms,msec     Simulated execution time in millisecs\n"
+	" us,usec     Simulated execution time in microseconds\n"
+	" ns,nsec     Simulated execution time in nanosecs\n"
+	" vclk        Number of virtual clock counts\n"
+	" fclk,fetch  Number of fetches\n"
+	" rclk,read   Number of memory read operations\n"
+	" wclk,write  Number of memory write operations\n")
+
 
 /*
  * Command: next
@@ -282,30 +295,42 @@ COMMAND_DO_WORK_SIM(cl_next_cmd)
 			      next, brkDYNAMIC, 1);
 
 	  b->init();
-//	  sim->uc->fbrk->add_bp(b);
-
 	  sim->uc->fbrk->add(b);
 	  b->activate();
 	}
-      /*if (sim->uc->fbrk_at(sim->uc->PC))
-	sim->uc->do_inst(1);*/
       sim->start(con, 0);
-      //sim->uc->do_inst(-1);
     }
   else {
-    //sim->uc->do_inst(1);
     sim->start(con, 1);
-    //sim->step();
-    //sim->stop(resSTEP);
-    //sim->uc->print_regs(con);
   }
   return(false);
 }
 
 CMDHELP(cl_next_cmd,
 	"next",
-	"Next",
-	"long help of next")
+	"Execute until next instruction is reached.",
+	"his command is similar to step command described above but\n"
+	"if actual instruction to execute is a subroutine call the next\n"
+	"command places a dynamic breakpoint after the call instruction\n"
+	"and starts to execute the subroutine. If the subroutine is\n"
+	"infinite the breakpoint set by next will never be reached.\n")
+
+/*
+ * Command: emulation
+ *----------------------------------------------------------------------------
+ */
+
+COMMAND_DO_WORK_SIM(cl_emu_cmd)
+{
+  sim->emulation(con);
+  return false;
+}
+
+CMDHELP(cl_emu_cmd,
+	"emulation",
+	"Start execution in emulation mode.",
+	"")
+
 
 /*
  * Command: help
@@ -358,7 +383,7 @@ COMMAND_DO_WORK_APP(cl_help_cmd)
 	      for (i= 0; i < subset->count; i++)
 		{
 		  class cl_cmd *c=
-		    dynamic_cast<class cl_cmd *>(subset->object_at(i));
+		    (class cl_cmd *)(subset->object_at(i));
 		  //con->dd_printf("%s\n", c->short_help.c_str());
 		  c->print_short(con);
 		}
@@ -376,7 +401,7 @@ COMMAND_DO_WORK_APP(cl_help_cmd)
 CMDHELP(cl_help_cmd,
 	"help [command [subcommand]]",
 	"List of known commands, or description of specified command",
-	"Long help of help command")
+	"")
 
 bool
 cl_help_cmd::do_set(class cl_cmdline *cmdline, int pari,
@@ -386,7 +411,7 @@ cl_help_cmd::do_set(class cl_cmdline *cmdline, int pari,
   int i;
   for (i= 0; i < cmdset->count; i++)
     {
-      class cl_cmd *cmd= dynamic_cast<class cl_cmd *>(cmdset->object_at(i));
+      class cl_cmd *cmd= (class cl_cmd *)(cmdset->object_at(i));
       if (!cmd)
 	continue;
       if (pari >= cmdline->nuof_params())
@@ -432,8 +457,10 @@ COMMAND_DO_WORK(cl_quit_cmd)
 
 CMDHELP(cl_quit_cmd,
 	"quit",
-	"Quit",
-	"long help of quit")
+	"Close actual console",
+	"Quit command closes the actual console. If -Z option was used\n"
+	"to start the simulator, it will continue running.\n"
+	"Otherwise it exists when last console closes.\n")
 
 /*
  * Command: kill
@@ -453,8 +480,8 @@ COMMAND_DO_WORK_APP(cl_kill_cmd)
 
 CMDHELP(cl_kill_cmd,
 	"kill",
-	"Shutdown simulator",
-	"long help of kill")
+	"Shutdown simulator.",
+	"")
 
 /*
  * EXEC file
@@ -483,7 +510,10 @@ COMMAND_DO_WORK_APP(cl_exec_cmd)
 CMDHELP(cl_exec_cmd,
 	"exec \"file\"",
 	"Execute commands from file",
-	"long help of exec")
+	"Reads commands from \"file\" and executes them. This command\n"
+	"opens a new console (which will use same in/out file as the\n"
+	"actual one) to execute the file. This means the \"quit\" command\n"
+	"in the file will not exit the simulator.")
 
 /*
  * expression expression
@@ -515,7 +545,10 @@ COMMAND_DO_WORK_APP(cl_expression_cmd)
 	  if (w.nempty())
 	    {
 	      v= application->eval(w);
-	      con->print_expr_result(v, fmt.nempty()?((const char *)fmt):NULL);
+	      if (fmt.nempty())
+		con->print_expr_result(v, fmt.c_str());
+	      else
+		con->print_expr_result(v, (const char *)NULL);
 	    }
 	  fmt= "";
 	}
@@ -542,11 +575,71 @@ CMDHELP(cl_expression_cmd,
 	" /B C style boolean value with 1|0: 1\n"
 	" /L logical boolean value with T|F: T\n"
 	" /c ascii character in C style escape: \'\\5137\'\n"
+	" /n print nothing\n"
 	"If more format characters are specified then result is printed in all\n"
 	"requested format.\n"
 	)
 
 
+/*
+ * ECHO command
+ *----------------------------------------------------------------------------
+ */
+
+COMMAND_DO_WORK_APP(cl_echo_cmd)
+{
+  class cl_cmd_arg *parm;
+  int i;
+  bool printed= false;
+  con->dd_color("answer");
+  for (i= 0; true; i++)
+    {
+      parm= cmdline->param(i);
+      if (!parm)
+	break;
+      if (!parm->as_string())
+	continue;
+      char *s= parm->value.string.string;
+      if (i)
+	con->dd_printf(" ");
+      con->dd_printf("%s", s);
+      printed= true;
+    }
+  if (printed)
+    con->dd_printf("\n");
+  return false;
+}
+
+CMDHELP(cl_echo_cmd,
+	"echo params...",
+	"Print parameters",
+	"");
+
+
+COMMAND_DO_WORK_APP(cl_dev_cmd)
+{
+  class cl_cmd_arg *parm;
+  int i;
+  con->dd_color("answer");
+  for (i= 0; true; i++)
+    {
+      parm= cmdline->param(i);
+      if (!parm)
+	break;
+      if (!parm->as_string())
+	continue;
+      char *s= parm->value.string.string;
+      class cl_console_sout *c= new cl_console_sout(app);
+      app->exec(chars(s), c);
+      printf("got: \"%s\"\n", c->sout.c_str());
+      delete c;
+    }
+  return false;
+}
+
+CMDHELP(cl_dev_cmd, "", "", "");
+
+
 /*
  * HISTORY command
  *----------------------------------------------------------------------------
@@ -565,7 +658,7 @@ COMMAND_DO_WORK_UC(cl_hist_cmd)
 CMDHELP(cl_hist_cmd,
 	"history",
 	"Execution history",
-	"long help of history")
+	"")
 
 
 /*
@@ -589,7 +682,7 @@ COMMAND_DO_WORK_UC(cl_hist_info_cmd)
 CMDHELP(cl_hist_info_cmd,
 	"history info",
 	"Information about execution history",
-	"long help of history info")
+	"")
 
 
 /*
@@ -612,7 +705,7 @@ COMMAND_DO_WORK_UC(cl_hist_clear_cmd)
 CMDHELP(cl_hist_clear_cmd,
 	"history clear",
 	"Clear execution history",
-	"long help of history clear")
+	"")
 
 
 /*
@@ -640,7 +733,7 @@ COMMAND_DO_WORK_UC(cl_hist_list_cmd)
 CMDHELP(cl_hist_list_cmd,
 	"history list [nr]",
 	"List last `nr' elements of execution history",
-	"long help of history list")
+	"")
 
 
 /* End of cmd.src/cmd_exec.cc */
