@@ -68,7 +68,7 @@ COMMAND_DO_WORK_UC(cl_gui_start_cmd)
 CMDHELP(cl_gui_start_cmd,
 	"gui start",
 	"Start interfacing with GUI tool",
-	"long help of gui start")
+	"")
 
 /*
  * Command: gui stop
@@ -86,6 +86,6 @@ COMMAND_DO_WORK_UC(cl_gui_stop_cmd)
 CMDHELP(cl_gui_stop_cmd,
 	"gui stop",
 	"Stop interfacing with GUI tool",
-	"long help of gui stop")
+	"")
 
 /* End of cmd.src/cmd_gui.cc */
