@@ -11,10 +11,20 @@
 # include HEADER_SOCKET
 #endif
 
+#include <stdio.h>
+#include <wchar.h>
 #include <windows.h>
+#ifdef HAVE_WINCON_H
+#include <wincon.h>
+#endif
+#ifdef HAVE_WINSOCK2_H
+#include <winsock2.h>
+#endif
 #include <io.h>
 #include <fcntl.h>
 #include <stdarg.h>
+#include <sys/time.h>
+#include <time.h>
 
 #include "fwiocl.h"
 
@@ -215,7 +225,7 @@ cl_io::check_dev(void)
         /*
          * Peek all pending console events
          */
-	//printf("win iput check on console id=%d handle=%p\n", file_id, handle);
+	//printf("win input check on console id=%d handle=%p\n", file_id, handle);
         if (INVALID_HANDLE_VALUE == handle)
 	  return ret;
 	if (!GetNumberOfConsoleInputEvents(handle, &NumPending))
@@ -346,6 +356,13 @@ cl_io::check(void)
     input_avail();
 }
 
+bool
+cl_io::writable(void)
+{
+  // TODO
+  return true;
+}
+
 void
 cl_io::changed(void)
 {
@@ -370,7 +387,7 @@ cl_io::changed(void)
   if (server_port > 0)
     {
       //printf("win opened socket id=%d\n", file_id);
-      handle= (void *)file_id;
+      handle= (void *)((ULONG_PTR)file_id);
       type= F_SOCKET;
       deb("assuming TTY on socket %d\n", file_id);
       tty= true;
@@ -414,7 +431,7 @@ cl_io::prepare_terminal()
       write(s, 7);
     }
 }
- 
+
 
 int
 mk_srv_socket(int port)
@@ -600,5 +617,103 @@ sigpipe_off()
 {
 }
 
+unsigned int cperiod_value() { return 10000; }
+
+int
+set_console_mode()
+{
+  // Set output mode to handle virtual terminal sequences
+  HANDLE hOut = GetStdHandle(STD_OUTPUT_HANDLE);
+  if (hOut == INVALID_HANDLE_VALUE)
+    {
+      return false;
+    }
+  HANDLE hIn = GetStdHandle(STD_INPUT_HANDLE);
+  if (hIn == INVALID_HANDLE_VALUE)
+    {
+      return false;
+    }
+  
+  DWORD dwOriginalOutMode = 0;
+  DWORD dwOriginalInMode = 0;
+  if (!GetConsoleMode(hOut, &dwOriginalOutMode))
+    {
+      return false;
+    }
+  if (!GetConsoleMode(hIn, &dwOriginalInMode))
+    {
+      return false;
+    }
+  
+  DWORD dwRequestedOutModes =
+#ifdef ENABLE_VIRTUAL_TERMINAL_PROCESSING
+    ENABLE_VIRTUAL_TERMINAL_PROCESSING
+#else
+    0x4
+#endif
+    |
+#ifdef DISABLE_NEWLINE_AUTO_RETURN
+    DISABLE_NEWLINE_AUTO_RETURN
+#else
+    0x8
+#endif
+    ;
+  //DWORD dwRequestedInModes = ENABLE_VIRTUAL_TERMINAL_INPUT;
+  
+  DWORD dwOutMode = dwOriginalOutMode | dwRequestedOutModes;
+  if (!SetConsoleMode(hOut, dwOutMode))
+    {
+      // we failed to set both modes, try to step down mode gracefully.
+      dwRequestedOutModes =
+#ifdef ENABLE_VIRTUAL_TERMINAL_PROCESSING
+	ENABLE_VIRTUAL_TERMINAL_PROCESSING
+#else
+	0x4
+#endif
+	;
+      dwOutMode = dwOriginalOutMode | dwRequestedOutModes;
+      if (!SetConsoleMode(hOut, dwOutMode))
+        {
+	  // Failed to set any VT mode, can't do anything here.
+	  return -1;
+        }
+    }
+  
+  DWORD dwInMode = dwOriginalInMode |
+#ifdef ENABLE_VIRTUAL_TERMINAL_INPUT
+    ENABLE_VIRTUAL_TERMINAL_INPUT
+#else
+    0x200
+#endif
+    ;
+  if (!SetConsoleMode(hIn, dwInMode))
+    {
+      // Failed to set VT input mode, can't do anything here.
+      return -1;
+    }
+  
+  return 0;
+}
+
+
+double
+dnow(void)
+{
+  struct timeval tv;
+  gettimeofday(&tv, NULL);
+  return (double)tv.tv_sec + ((double)tv.tv_usec/1000000.0);
+}
+
+
+void
+save_std_attribs()
+{
+}
+
+void
+restore_std_attribs()
+{
+}
+
 
 /* End of fwio.cc */
