@@ -28,6 +28,8 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 // local
 #include "simp1516cl.h"
 #include "p1516cl.h"
+#include "p2223cl.h"
+#include "glob.h"
 
 
 cl_simp1516::cl_simp1516(class cl_app *the_app):
@@ -37,7 +39,38 @@ cl_simp1516::cl_simp1516(class cl_app *the_app):
 class cl_uc *
 cl_simp1516::mk_controller(void)
 {
-  return(new cl_p1516(this));
+  int i;
+  const char *typ= 0;
+  class cl_optref type_option(this);
+  class cl_p1516 *uc;
+
+  type_option.init();
+  type_option.use("cpu_type");
+  i= 0;
+  if ((typ= type_option.get_value(typ)) == 0)
+    typ= "P2223";
+  while ((cpus_p1516[i].type_str != NULL) &&
+	 (strcasecmp(typ, cpus_p1516[i].type_str) != 0))
+    i++;
+  if (cpus_p1516[i].type_str == NULL)
+    {
+      fprintf(stderr, "Unknown processor type. "
+	      "Use -H option to see known types.\n");
+      return(NULL);
+    }
+  switch (cpus_p1516[i].type)
+    {
+    case CPU_P1516:
+      uc= new cl_p1516(this);
+      return uc;
+    case CPU_P2223:
+      uc= new cl_p2223(this);
+      return uc;
+    default:
+      fprintf(stderr, "Unknown processor type\n");
+      return NULL;
+    }
+  return NULL;
 }
 
 
