@@ -58,13 +58,20 @@ int cl_pdk::get_mem(unsigned int addr) {
 }
 
 unsigned char cl_pdk::get_io(t_addr addr) {
-  return regs8->get(addr);
+  return regs8->read(addr);
 }
 
-void cl_pdk::store_io(t_addr addr, unsigned char value) {
-  regs8->set(addr, value);
-  if (get_SP() > sp_max)
-    sp_max = get_SP();
+int cl_pdk::store_io(t_addr addr, int value) {
+  
+  regs8->write(addr, value & 0xFF);
+  if (addr == 0x02)
+    {
+      if (get_SP() > sp_most)
+        sp_most = get_SP();
+      if (!ram->valid_address(value))
+        return resSTACK_OV;
+    }
+  return resGO;
 }
 
 unsigned char cl_pdk::get_SP() {
@@ -76,7 +83,7 @@ unsigned char cl_pdk::get_flags() {
 }
 
 void cl_pdk::set_flags(unsigned char flags) {
-    return store_io(0x00, flags);
+    store_io(0x00, flags);
 }
 
 int cl_pdk::get_flag(flag n) {
@@ -92,6 +99,7 @@ int cl_pdk::get_flag(flag n) {
 }
 
 void cl_pdk::store_flag(flag n, int value) {
+  value= value?1:0;
   switch (n) {
   case flag_z: set_flags((get_flags() & ~1) | value); break;
   case flag_c: set_flags((get_flags() & ~2) | (value << 1)); break;
@@ -111,28 +119,31 @@ int cl_pdk::execute(unsigned int code) {
   case CPU_PDK15:
     return(execute_pdk15(code));
   default:
-    __builtin_unreachable();
+    return 0;//__builtin_unreachable();
   }
 }
 
 int cl_pdk::execute_pdk14(unsigned int code) {
+  int write_result = resGO;
   if (code == 0x0000) {
     // nop
   } else if (CODE_MASK(0x0200, 0xFF)) {
     // ret k
     regs.a = code & 0xFF;
-    store_io(0x2, get_SP() - 2);
-    PC = get_mem(get_SP()) | (get_mem(get_SP() + 1) << 8);
+    write_result = store_io(0x2, get_SP() - 2);
+    if (write_result == resGO)
+      PC = get_mem(get_SP()) | (get_mem(get_SP() + 1) << 8);
   } else if (code == 0x007A) {
     // ret
-    store_io(0x2, get_SP() - 2);
-    PC = get_mem(get_SP()) | (get_mem(get_SP() + 1) << 8);
+    write_result = store_io(0x2, get_SP() - 2);
+    if (write_result == resGO)
+      PC = get_mem(get_SP()) | (get_mem(get_SP() + 1) << 8);
   } else if (CODE_MASK(0x2F00, 0xFF)) {
     // mov a, k
     regs.a = code & 0xFF;
   } else if (CODE_MASK(0x0180, 0x3F)) {
     // mov i, a
-    store_io(code & 0x3F, regs.a);
+    write_result = store_io(code & 0x3F, regs.a);
   } else if (CODE_MASK(0x01C0, 0x3F)) {
     // mov a, i
     regs.a = get_io(code & 0x3F);
@@ -161,12 +172,12 @@ int cl_pdk::execute_pdk14(unsigned int code) {
     // pushaf
     ram->write(get_SP(), regs.a);
     ram->write(get_SP() + 1, get_flags());
-    store_io(0x2, get_SP() + 2);
+    write_result = store_io(0x2, get_SP() + 2);
   } else if (code == 0x0073) {
     // popaf
     set_flags(get_mem(get_SP() - 1));
     regs.a = get_mem(get_SP() - 2);
-    store_io(0x2, get_SP() - 2);
+    write_result = store_io(0x2, get_SP() - 2);
   } else if (CODE_MASK(0x2800, 0xFF)) {
     // add a, k
     regs.a = add_to(regs.a, code & 0xFF);
@@ -309,24 +320,30 @@ int cl_pdk::execute_pdk14(unsigned int code) {
     ram->write(code & 0x7F, store);
   } else if (CODE_MASK(0x00C0, 0x3F)) {
     // xor io, a
-    store_io(code & 0x3F, regs.a ^ get_io(code & 0x3F));
+    write_result = store_io(code & 0x3F, regs.a ^ get_io(code & 0x3F));
   } else if (code == 0x0068) {
     // not a
     regs.a = ~regs.a;
+    store_flag(flag_z, !regs.a);
   } else if (CODE_MASK(0x1400, 0x7F)) {
     // not m
-    ram->write(code & 0x7F, ~get_mem(code & 0x7F));
+    int store = (~get_mem(code & 0x7F) & 0xff);
+    store_flag(flag_z, !store);
+    ram->write(code & 0x7F, store);
   } else if (code == 0x0069) {
     // neg a
     regs.a = -regs.a;
+    store_flag(flag_z, !regs.a);
   } else if (CODE_MASK(0x1480, 0x7F)) {
     // neg m
-    ram->write(code & 0x7F, -get_mem(code & 0x7F));
+    int store = (-get_mem(code & 0x7F) & 0xff);
+    store_flag(flag_z, !store);
+    ram->write(code & 0x7F, store);
   } else if (CODE_MASK(0x1C00, 0x1FF)) {
     // set0 io, k
     const u8_t bit = (code & 0x1C0) >> 6;
     const u8_t addr = code & 0x3F;
-    store_io(addr, get_io(addr) & ~(1 << bit));
+    write_result = store_io(addr, get_io(addr) & ~(1 << bit));
   } else if (CODE_MASK(0x2400, 0x1FF)) {
     // set0 m, k
     const u8_t bit = (code & 0x1C0) >> 6;
@@ -336,7 +353,7 @@ int cl_pdk::execute_pdk14(unsigned int code) {
     // set1 io, k
     const u8_t bit = (code & 0x1C0) >> 6;
     const u8_t addr = code & 0x3F;
-    store_io(addr, get_io(addr) | (1 << bit));
+    write_result = store_io(addr, get_io(addr) | (1 << bit));
   } else if (CODE_MASK(0x2600, 0x1FF)) {
     // set1 m, k
     const u8_t bit = (code & 0x1C0) >> 6;
@@ -413,7 +430,7 @@ int cl_pdk::execute_pdk14(unsigned int code) {
     ram->write(get_SP(), PC);
     ram->write(get_SP() + 1, PC >> 8);
     PC = code & 0x7FF;
-    store_io(0x2, get_SP() + 2);
+    write_result = store_io(0x2, get_SP() + 2);
   } else if (CODE_MASK(0x3000, 0x7FF)) {
     // goto k
     PC = code & 0x7FF;
@@ -458,7 +475,7 @@ int cl_pdk::execute_pdk14(unsigned int code) {
     // mul
     unsigned result = regs.a * get_io(0x08);
     regs.a = result & 0xFF;
-    store_io(0x08, (result & 0xFF00) >> 8);
+    write_result = store_io(0x08, (result & 0xFF00) >> 8);
   } else if (code == 0xFF00) {
     // putchar - usim specific instruction
     putchar(regs.a);
@@ -466,27 +483,30 @@ int cl_pdk::execute_pdk14(unsigned int code) {
   } else {
     return (resINV_INST);
   }
-  return (resGO);
+  return (write_result);
 }
 
 int cl_pdk::execute_pdk13(unsigned int code) {
+  int write_result = resGO;
   if (code == 0x0000) {
     // nop
   } else if (CODE_MASK(0x0100, 0xFF)) {
     // ret k
     regs.a = code & 0xFF;
-    store_io(0x2, get_SP() - 2);
-    PC = get_mem(get_SP()) | (get_mem(get_SP() + 1) << 8);
+    write_result = store_io(0x2, get_SP() - 2);
+    if (write_result == resGO)
+      PC = get_mem(get_SP()) | (get_mem(get_SP() + 1) << 8);
   } else if (code == 0x003A) {
     // ret
-    store_io(0x2, get_SP() - 2);
-    PC = get_mem(get_SP()) | (get_mem(get_SP() + 1) << 8);
+    write_result = store_io(0x2, get_SP() - 2);
+    if (write_result == resGO)
+      PC = get_mem(get_SP()) | (get_mem(get_SP() + 1) << 8);
   } else if (CODE_MASK(0x1700, 0xFF)) {
     // mov a, k
     regs.a = code & 0xFF;
   } else if (CODE_MASK(0x0080, 0x1F)) {
     // mov i, a
-    store_io(code & 0x1F, regs.a);
+    write_result = store_io(code & 0x1F, regs.a);
   } else if (CODE_MASK(0x00A0, 0x1F)) {
     // mov a, i
     regs.a = get_io(code & 0x1F);
@@ -498,7 +518,7 @@ int cl_pdk::execute_pdk13(unsigned int code) {
     regs.a = get_mem(code & 0x3F);
   } else if (CODE_MASK(0x00C1, 0x1E)) {
     // TODO: ldt16
-  } else if (CODE_MASK(0x0C00, 0x1E)) {
+  } else if (CODE_MASK(0x00C0, 0x1E)) {
     // TODO: stt16
   } else if ((CODE_MASK(0x0E1, 0x1E))) {
     // idxm a, m
@@ -515,12 +535,12 @@ int cl_pdk::execute_pdk13(unsigned int code) {
     // pushaf
     ram->write(get_SP(), regs.a);
     ram->write(get_SP() + 1, get_flags());
-    store_io(0x2, get_SP() + 2);
+    write_result = store_io(0x2, get_SP() + 2);
   } else if (code == 0x0033) {
     // popaf
     set_flags(get_mem(get_SP() - 1));
     regs.a = get_mem(get_SP() - 2);
-    store_io(0x2, get_SP() - 2);
+    write_result = store_io(0x2, get_SP() - 2);
   } else if (CODE_MASK(0x1000, 0xFF)) {
     // add a, k
     regs.a = add_to(regs.a, code & 0xFF);
@@ -663,58 +683,64 @@ int cl_pdk::execute_pdk13(unsigned int code) {
     ram->write(code & 0x3F, store);
   } else if (CODE_MASK(0x0060, 0x1F)) {
     // xor io, a
-    store_io(code & 0x1F, regs.a ^ get_io(code & 0x1F));
+    write_result = store_io(code & 0x1F, regs.a ^ get_io(code & 0x1F));
   } else if (code == 0x0018) {
     // not a
     regs.a = ~regs.a;
+    store_flag(flag_z, !regs.a);
   } else if (CODE_MASK(0x0A00, 0x3F)) {
     // not m
-    ram->write(code & 0x3F, ~get_mem(code & 0x3F));
+    int store = (~get_mem(code & 0x3F) & 0xff);
+    store_flag(flag_z, !store);
+    ram->write(code & 0x3F, store);
   } else if (code == 0x0019) {
     // neg a
     regs.a = -regs.a;
+    store_flag(flag_z, !regs.a);
   } else if (CODE_MASK(0x0A40, 0x3F)) {
     // neg m
-    ram->write(code & 0x3F, -get_mem(code & 0x3F));
+    int store = (-get_mem(code & 0x3F) & 0xff);
+    store_flag(flag_z, !store);
+    ram->write(code & 0x3F, store);
   } else if (CODE_MASK(0x0E00, 0xFF)) {
     // set0 io, k
     const u8_t bit = (code & 0xE0) >> 5;
     const u8_t addr = code & 0x1F;
-    store_io(addr, get_io(addr) & ~(1 << bit));
-  } else if (CODE_MASK(0x0300, 0xFE)) {
+    write_result = store_io(addr, get_io(addr) & ~(1 << bit));
+  } else if (CODE_MASK(0x0300, 0xEF)) {
     // set0 m, k
     const u8_t bit = (code & 0xE0) >> 5;
-    const u8_t addr = (code & 0xFE) >> 1;
+    const u8_t addr = code & 0x0F;
     ram->write(addr, get_mem(addr) & ~(1 << bit));
   } else if (CODE_MASK(0x0F00, 0xFF)) {
     // set1 io, k
     const u8_t bit = (code & 0xE0) >> 5;
     const u8_t addr = code & 0x1F;
-    store_io(addr, get_io(addr) | (1 << bit));
-  } else if (CODE_MASK(0x0301, 0xFE)) {
+    write_result = store_io(addr, get_io(addr) | (1 << bit));
+  } else if (CODE_MASK(0x0310, 0xEF)) {
     // set1 m, k
     const u8_t bit = (code & 0xE0) >> 5;
-    const u8_t addr = (code & 0x1E) >> 1;
+    const u8_t addr = code & 0x0F;
     ram->write(addr, get_mem(addr) | (1 << bit));
   } else if (CODE_MASK(0x0C00, 0xFF)) {
     // t0sn io, k
     int n = (code & 0xE0) >> 5;
     if (!(get_io(code & 0x1F) & (1 << n)))
       ++PC;
-  } else if (CODE_MASK(0x0200, 0xFE)) {
+  } else if (CODE_MASK(0x0200, 0xEF)) {
     // t0sn m, k
     int n = (code & 0xE0) >> 5;
-    if (!(get_mem((code & 0xFE) >> 1) & (1 << n)))
+    if (!(get_mem(code & 0x0F) & (1 << n)))
       ++PC;
   } else if (CODE_MASK(0x0D00, 0xFF)) {
     // t1sn io, k
     int n = (code & 0xE0) >> 5;
     if (get_io(code & 0x1F) & (1 << n))
       ++PC;
-  } else if (CODE_MASK(0x0201, 0xFE)) {
+  } else if (CODE_MASK(0x0210, 0xEF)) {
     // t1sn m, k
     int n = (code & 0xE0) >> 5;
-    if (get_mem((code & 0xFE) >> 1) & (1 << n))
+    if (get_mem(code & 0x0F) & (1 << n))
       ++PC;
   } else if (CODE_MASK(0x1200, 0xFF)) {
     // ceqsn a, k
@@ -756,7 +782,7 @@ int cl_pdk::execute_pdk13(unsigned int code) {
     ram->write(get_SP(), PC);
     ram->write(get_SP() + 1, PC >> 8);
     PC = code & 0x3FF;
-    store_io(0x2, get_SP() + 2);
+    write_result = store_io(0x2, get_SP() + 2);
   } else if (CODE_MASK(0x1800, 0x3FF)) {
     // goto k
     PC = code & 0x3FF;
@@ -788,7 +814,7 @@ int cl_pdk::execute_pdk13(unsigned int code) {
     // mul
     unsigned result = regs.a * get_io(0x08);
     regs.a = result & 0xFF;
-    store_io(0x08, (result & 0xFF00) >> 8);
+    write_result = store_io(0x08, (result & 0xFF00) >> 8);
   } else if (code == 0xFF00) {
     // putchar - usim specific instruction
     putchar(regs.a);
@@ -796,27 +822,30 @@ int cl_pdk::execute_pdk13(unsigned int code) {
   } else {
     return (resINV_INST);
   }
-  return (resGO);
+  return (write_result);
 }
 
 int cl_pdk::execute_pdk15(unsigned int code) {
+  int write_result = resGO;
   if (code == 0x0000) {
     // nop
   } else if (CODE_MASK(0x0200, 0xFF)) {
     // ret k
     regs.a = code & 0xFF;
-    store_io(0x2, get_SP() - 2);
-    PC = get_mem(get_SP()) | (get_mem(get_SP() + 1) << 8);
+    write_result = store_io(0x2, get_SP() - 2);
+    if (write_result == resGO)
+      PC = get_mem(get_SP()) | (get_mem(get_SP() + 1) << 8);
   } else if (code == 0x007A) {
     // ret
-    store_io(0x2, get_SP() - 2);
-    PC = get_mem(get_SP()) | (get_mem(get_SP() + 1) << 8);
+    write_result = store_io(0x2, get_SP() - 2);
+    if (write_result == resGO)
+      PC = get_mem(get_SP()) | (get_mem(get_SP() + 1) << 8);
   } else if (CODE_MASK(0x5700, 0xFF)) {
     // mov a, k
     regs.a = code & 0xFF;
   } else if (CODE_MASK(0x0100, 0x7F)) {
     // mov i, a
-    store_io(code & 0x7F, regs.a);
+    write_result = store_io(code & 0x7F, regs.a);
   } else if (CODE_MASK(0x0180, 0x7F)) {
     // mov a, i
     regs.a = get_io(code & 0x7F);
@@ -845,12 +874,12 @@ int cl_pdk::execute_pdk15(unsigned int code) {
     // pushaf
     ram->write(get_SP(), regs.a);
     ram->write(get_SP() + 1, get_flags());
-    store_io(0x2, get_SP() + 2);
+    write_result = store_io(0x2, get_SP() + 2);
   } else if (code == 0x0073) {
     // popaf
     set_flags(get_mem(get_SP() - 1));
     regs.a = get_mem(get_SP() - 2);
-    store_io(0x2, get_SP() - 2);
+    write_result = store_io(0x2, get_SP() - 2);
   } else if (CODE_MASK(0x5000, 0xFF)) {
     // add a, k
     regs.a = add_to(regs.a, code & 0xFF);
@@ -993,24 +1022,30 @@ int cl_pdk::execute_pdk15(unsigned int code) {
     ram->write(code & 0xFF, store);
   } else if (CODE_MASK(0x0080, 0x7F)) {
     // xor io, a
-    store_io(code & 0x3F, regs.a ^ get_io(code & 0x3F));
+    write_result = store_io(code & 0x3F, regs.a ^ get_io(code & 0x3F));
   } else if (code == 0x0068) {
     // not a
     regs.a = ~regs.a;
+    store_flag(flag_z, !regs.a);
   } else if (CODE_MASK(0x2800, 0xFF)) {
     // not m
-    ram->write(code & 0xFF, ~get_mem(code & 0xFF));
+    int store = (~get_mem(code & 0xFF) & 0xFF);
+    store_flag(flag_z, !store);
+    ram->write(code & 0xFF, store);
   } else if (code == 0x0069) {
     // neg a
     regs.a = -regs.a;
+    store_flag(flag_z, !regs.a);
   } else if (CODE_MASK(0x2900, 0xFF)) {
     // neg m
-    ram->write(code & 0xFF, -get_mem(code & 0xFF));
+    int store = (-get_mem(code & 0xFF) & 0xFF);
+    store_flag(flag_z, !store);
+    ram->write(code & 0xFF, store);
   } else if (CODE_MASK(0x3800, 0x3FF)) {
     // set0 io, k
     const u8_t bit = (code & 0x380) >> 7;
     const u8_t addr = code & 0x7F;
-    store_io(addr, get_io(addr) & ~(1 << bit));
+    write_result = store_io(addr, get_io(addr) & ~(1 << bit));
   } else if (CODE_MASK(0x4800, 0x3FF)) {
     // set0 m, k
     const u8_t bit = (code & 0x380) >> 7;
@@ -1020,7 +1055,7 @@ int cl_pdk::execute_pdk15(unsigned int code) {
     // set1 io, k
     const u8_t bit = (code & 0x380) >> 7;
     const u8_t addr = code & 0x7F;
-    store_io(addr, get_io(addr) | (1 << bit));
+    write_result = store_io(addr, get_io(addr) | (1 << bit));
   } else if (CODE_MASK(0x4C00, 0x3FF)) {
     // set1 m, k
     const u8_t bit = (code & 0x380) >> 7;
@@ -1097,7 +1132,7 @@ int cl_pdk::execute_pdk15(unsigned int code) {
     ram->write(get_SP(), PC);
     ram->write(get_SP() + 1, PC >> 8);
     PC = code & 0xFFF;
-    store_io(0x2, get_SP() + 2);
+    write_result = store_io(0x2, get_SP() + 2);
   } else if (CODE_MASK(0x6000, 0xFFF)) {
     // goto k
     PC = code & 0xFFF;
@@ -1142,7 +1177,7 @@ int cl_pdk::execute_pdk15(unsigned int code) {
     // mul
     unsigned result = regs.a * get_io(0x08);
     regs.a = result & 0xFF;
-    store_io(0x08, (result & 0xFF00) >> 8);
+    write_result = store_io(0x08, (result & 0xFF00) >> 8);
   } else if (code == 0xFF00) {
     // putchar - usim specific instruction
     putchar(regs.a);
@@ -1150,8 +1185,8 @@ int cl_pdk::execute_pdk15(unsigned int code) {
   } else {
     return (resINV_INST);
   }
-  return (resGO);
+  return (write_result);
 }
 
 /* End of pdk.src/inst.cc */
-/* End of pdk.src/inst.cc */
+
