@@ -57,13 +57,18 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 
 cl_pdk::cl_pdk(struct cpu_entry *IType, class cl_sim *asim) : cl_uc(asim) {
   type = IType;
+  if (type->type == CPU_PDK13)
+    PCmask= 0x3ff;
+  if (type->type == CPU_PDK14)
+    PCmask= 0x7ff;
+  if (type->type == CPU_PDK15)
+    PCmask= 0xfff;
 }
 
 int cl_pdk::init(void) {
   cl_uc::init(); /* Memories now exist */
 
-  set_xtal(8000000);
-  sp_max = 0x00;
+  //set_xtal(8000000);
 
   // rom = address_space(MEM_ROM_ID);
   // ram = mem(MEM_XRAM);
@@ -79,6 +84,7 @@ int cl_pdk::init(void) {
 
 void cl_pdk::reset(void) {
   cl_uc::reset();
+  sp_most = 0x00;
 
   PC = 0x0000;
   regs.a = 0;
@@ -167,7 +173,25 @@ void cl_pdk::make_memories(void) {
     chip->init();
     memchips->add(chip);
 
-    ad = new cl_address_decoder(as = address_space("rom"), chip, 0, rom_storage, 0);
+    ad = new cl_address_decoder(as = address_space("rom"), chip, 0, rom_storage-1, 0);
+    ad->init();
+    as->decoders->add(ad);
+    ad->activate(0);
+
+    chip = new cl_chip16("ram_chip", ram_storage, 8);
+    chip->init();
+    memchips->add(chip);
+
+    ad = new cl_address_decoder(as = address_space("ram"), chip, 0, ram_storage-1, 0);
+    ad->init();
+    as->decoders->add(ad);
+    ad->activate(0);
+
+    chip = new cl_chip16("io_chip", io_size, 8);
+    chip->init();
+    memchips->add(chip);
+
+    ad = new cl_address_decoder(as = address_space("regs8"), chip, 0, io_size-1, 0);
     ad->init();
     as->decoders->add(ad);
     ad->activate(0);
@@ -402,21 +426,22 @@ void cl_pdk::print_regs(class cl_console_base *con) {
  * Execution
  */
 
-int cl_pdk::exec_inst(void) {
+int cl_pdk::exec_inst(void)
+{
   t_mem code;
 
+  instPC= PC;
   if (fetch(&code)) {
     return (resBREAKPOINT);
   }
   tick(1);
 
   int status = execute(code);
-  if (status == resINV_INST) {
-    PC = rom->inc_address(PC, -1);
-
-    sim->stop(resINV_INST);
-    return (resINV_INST);
-  }
+  if (status == resINV_INST)
+    {
+      //PC = instPC;//rom->inc_address(PC, -1);
+      return (resINV_INST);
+    }
   return (status);
 }
 
