@@ -78,7 +78,7 @@ cl_rxk::init(void)
   fill_dd_wrappers(itab_dd);
   fill_ed_wrappers(itab_ed);
   
-  set_xtal(1000000);
+  //set_xtal(1000000);
 
 #define RCV(R) reg_cell_var(&c ## R , &r ## R , "" #R "" , "CPU register " #R "")
   RCV(A);
@@ -662,22 +662,28 @@ cl_rxk::print_regs(class cl_console_base *con)
 		 mem->get_xpc(), rIP, rIIR, rEIR);
   
   con->dd_printf("BC= ");
-  rom->dump(0, rBC, rBC+7, 8, con);
+  class cl_dump_ads ads(rBC, rBC+7);
+  rom->dump(0, /*rBC, rBC+7*/&ads, 8, con);
   con->dd_color("answer");
   con->dd_printf("DE= ");
-  rom->dump(0, rDE, rDE+7, 8, con);
+  ads._ss(rDE, rDE+7);
+  rom->dump(0, /*rDE, rDE+7*/&ads, 8, con);
   con->dd_color("answer");
   con->dd_printf("HL= ");
-  rom->dump(0, rHL, rHL+7, 8, con);
+  ads._ss(rHL, rHL+7);
+  rom->dump(0, /*rHL, rHL+7*/&ads, 8, con);
   con->dd_color("answer");
   con->dd_printf("IX= ");
-  rom->dump(0, rIX, rIX+7, 8, con);
+  ads._ss(rIX, rIX+7);
+  rom->dump(0, /*rIX, rIX+7*/&ads, 8, con);
   con->dd_color("answer");
   con->dd_printf("IY= ");
-  rom->dump(0, rIY, rIY+7, 8, con);
+  ads._ss(rIY, rIY+7);
+  rom->dump(0, /*rIY, rIY+7*/&ads, 8, con);
   con->dd_color("answer");
   con->dd_printf("SP= ");
-  rom->dump(0, rSP, rSP+7, 8, con);
+  ads._ss(rSP, rSP+7);
+  rom->dump(0, /*rSP, rSP+7*/&ads, 8, con);
   con->dd_color("answer");
 
   con->dd_printf("aAF= 0x%02x-0x%02x  ", raA, raF);
@@ -958,7 +964,7 @@ cl_rxk_cpu::print_info(class cl_console_base *con)
       else if (la >= (y<<12)) t= 'D', con->dd_color("dump_label");
       else t= 'C', con->dd_color("answer");
       c= ruc->mem->get_cell(la);
-      pa= ruc->mem->chip->is_slot(c->get_data());
+      ruc->mem->chip->is_slot(c->get_data(), &pa);
       con->dd_printf("0x%xxxx -> 0x%03xxxx %c", l, pa>>12, t);
       con->dd_printf("   ");
       la= r*0x1000;
@@ -967,7 +973,7 @@ cl_rxk_cpu::print_info(class cl_console_base *con)
       else if (la >= (y<<12)) t= 'D', con->dd_color("dump_label");
       else t= 'C', con->dd_color("answer");
       c= ruc->mem->get_cell(la);
-      pa= ruc->mem->chip->is_slot(c->get_data());
+      ruc->mem->chip->is_slot(c->get_data(), &pa);
       con->dd_printf("0x%xxxx -> 0x%03xxxx %c", r, pa>>12, t);
       con->dd_color("answer");
       con->dd_printf("\n");
