@@ -18,14 +18,20 @@
 #define BITPOS_I1 5  // 20H
 #define BITPOS_V 7  // 80H
 
+#define store2(addr, val) { ram->write((t_addr) (addr), (val >> 8) & 0xff); \
+                            ram->write((t_addr) (addr+1), val & 0xff); \
+  			    vc.wr+= 2; }
+#define store1(addr, val) { ram->write((t_addr) (addr), val); vc.wr++; }
 //#define get1(addr) ram->read((t_addr) (addr))
 #define get1(addr) get_1(addr)
 #define fetch2() ((fetch() << 8) | fetch() )
 #define fetch1() fetch()
 #define push2(val) {							\
     t_addr sp_before= regs.SP;						\
-    store2(regs.SP-1,(val));						\
-    regs.SP-=2;								\
+    store1(regs.SP,(val));						\
+    cSP.W(regs.SP-1);							\
+    store1(regs.SP,(val)>>8);						\
+    cSP.W(regs.SP-1);							\
     class cl_stack_op *so=						\
       new cl_stack_push(instPC,val,sp_before,regs.SP);			\
     so->init();								\
@@ -34,14 +40,14 @@
 #define push1(val) {							\
     t_addr sp_before= regs.SP;						\
     store1(regs.SP,(val));						\
-    regs.SP-=1;								\
+    cSP.W(regs.SP-1);							\
     class cl_stack_op *so=						\
       new cl_stack_push(instPC,val,sp_before,regs.SP);			\
     so->init();								\
     stack_write(so);							\
   }
-#define pop2(var) {var=get2(regs.SP+1); regs.SP+=2;}
-#define pop1(var) {var=get1(regs.SP+1); regs.SP+=1;}
+#define pop2(var) {u8_t h,l;cSP.W(regs.SP+1);h=get1(regs.SP);cSP.W(regs.SP+1);l=get1(regs.SP);var=h*256+l;}
+#define pop1(var) {cSP.W(regs.SP+1); var=get1(regs.SP);}
 
 
 #define FLAG_SET(f) {regs.CC |= f;}
