@@ -56,21 +56,29 @@ extern const char *object_name(class cl_base *o);
 extern char *case_string(enum letter_case lcase, const char *str);
 extern chars cbin(long data, int bits);
 
-extern double dnow(void);
-
 extern int strispn(char *s, char c);
 extern bool strend(const char *search_in, const char *what);
 extern bool valid_sym_name(char *s);
-
+extern bool filename_has_ext(class cl_f *f, const char *ext);
 extern bool is_hex_file(class cl_f *f);
 extern bool is_asc_file(class cl_f *f);
+extern bool is_p2h_file(class cl_f *f);
 extern bool is_omf_file(class cl_f *f);
 extern bool is_cdb_file(class cl_f *f);
+extern bool is_s19_file(class cl_f *f);
+extern bool is_map_file(class cl_f *f);
 
 extern chars colopt2ansiseq(char *opt);
 
 extern double strtoscale(const char *s, const char **units = 0);
 
+extern void srnd(unsigned int seed);
+extern unsigned int urnd(void);
+extern u8_t urnd8(void);
+extern u16_t urnd16(void);
+extern u32_t urnd32(void);
+
+
 #endif
 
 /* End of utils.h */
