@@ -27,6 +27,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 
 // prj
 #include "globals.h"
+#include "utils.h"
 
 // sim.src
 //#include "appcl.h"
@@ -40,6 +41,7 @@ main(int argc, char *argv[])
 {
   class cl_sim *sim;
 
+  app_start_at= dnow();
   application= new cl_app();
   application->set_name("sxa");
   application->init(argc, argv);
