@@ -32,6 +32,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 
 extern struct dis_entry disass_z80[];
 
+extern struct dis_entry disass_z80n_ed[];
 extern struct dis_entry disass_z80_ed[];
 extern struct dis_entry disass_z80_cb[];
 extern struct dis_entry disass_z80_dd[];
@@ -39,18 +40,10 @@ extern struct dis_entry disass_z80_fd[];
 extern struct dis_entry disass_z80_ddcb[];
 extern struct dis_entry disass_z80_fdcb[];
 
+extern struct dis_entry disass_gb80[];
+extern struct dis_entry disass_gb80_cb[];
 
-extern struct dis_entry disass_r2k[];
-
-extern struct dis_entry disass_r2k_ed[];
-extern struct dis_entry disass_r2k_cb[];
-extern struct dis_entry disass_r2k_dd[];
-extern struct dis_entry disass_r2k_fd[];
-extern struct dis_entry disass_r2k_ddcb[];
-extern struct dis_entry disass_r2k_fdcb[];
-
-extern struct dis_entry disass_lr35902[];
-extern struct dis_entry disass_lr35902_cb[];
+extern struct cpu_entry cpus_z80[];
 
 #endif
 
