@@ -124,12 +124,15 @@ int  cl_z80::inst_ed_(t_mem code)
       
     case 0x45: // RETN (return from non-maskable interrupt)
       pop2(PC);
-      vc.rd+= 2;
+      IFF1= IFF2;
+      IFF2= false;
+      vc.rd+= 2;      
       tick(13);
       return(resGO);
 
     case 0x46: // IM 0
       /* interrupt device puts opcode on data bus */
+      imode= 0;
       tick(7);
       return(resGO);
 
@@ -172,6 +175,8 @@ int  cl_z80::inst_ed_(t_mem code)
       
     case 0x4D: // RETI (return from interrupt)
       pop2(PC);
+      IFF1= IFF2;
+      IFF2= false;
       vc.rd+= 2;
       tick(13);
       return(resGO);
@@ -208,6 +213,7 @@ int  cl_z80::inst_ed_(t_mem code)
       return(resGO);
 
     case 0x56: // IM 1
+      imode= 1;
       tick(7);
       return(resGO);
 
@@ -253,6 +259,7 @@ int  cl_z80::inst_ed_(t_mem code)
       return(resGO);
 
     case 0x5E: // IM 2
+      imode= 2;
       tick(7);
       return(resGO);
 
@@ -405,7 +412,7 @@ int  cl_z80::inst_ed_(t_mem code)
       return(resGO);
 
     case 0xA0: // LDI
-      // BC - count, sourc=HL, dest=DE.  *DE++ = *HL++, --BC until zero
+      // BC - count, source=HL, dest=DE.  *DE++ = *HL++, --BC until zero
       regs.raf.F &= ~(BIT_P | BIT_N | BIT_A);  /* clear these */
       store1(regs.DE, get1(regs.HL));
       ++regs.HL;
@@ -511,19 +518,25 @@ int  cl_z80::inst_ed_(t_mem code)
       return(resGO);
 
     case 0xB0: // LDIR
-      // BC - count, sourc=HL, dest=DE.  *DE++ = *HL++, --BC until zero
-      regs.raf.F &= ~(BIT_P | BIT_N | BIT_A);  /* clear these */
-      do {
-        store1(regs.DE, get1(regs.HL));
-        ++regs.HL;
-        ++regs.DE;
-        --regs.BC;
-        vc.rd++;
-        vc.wr++;
-	tick(15);
-	if (regs.BC) tick(5);
-      } while (regs.BC != 0);
-      return(resGO);
+      // BC - count, source=HL, dest=DE.  *DE++ = *HL++, --BC until zero
+      {
+	bool first= true;
+	regs.raf.F &= ~(BIT_P | BIT_N | BIT_A);  /* clear these */
+	do {
+	  store1(regs.DE, get1(regs.HL));
+	  ++regs.HL;
+	  ++regs.DE;
+	  --regs.BC;
+	  vc.rd++;
+	  vc.wr++;
+	  if (first)
+	    tick(15), first= false;
+	  else
+	    tick(16);
+	  if (regs.BC) tick(5);
+	} while (regs.BC != 0);
+	return(resGO);
+      }
 
     case 0xB1: // CPIR
       // compare acc with mem(HL), if ACC=0 set Z flag.  Incr HL, decr BC.
@@ -652,6 +665,45 @@ int  cl_z80::inst_ed_(t_mem code)
       while (regs.BC);
       return(resGO);
 
+    // Z280/R800 multu
+    case 0xc1:
+    case 0xc9:
+    case 0xd1:
+    case 0xd9:
+      if (type->type==CPU_R800)
+        {
+          unsigned int result = (unsigned int)(regs.raf.A) * reg_g_read((code >> 3) & 0x07);
+          regs.HL = result;
+          regs.raf.F &= ~(BIT_S | BIT_Z | BIT_P | BIT_C);
+          if (!result)
+            regs.raf.F |= BIT_Z;
+          if (result >= 0x100)
+            regs.raf.F |= BIT_C;
+          tick (14);
+          return(resGO);
+        }
+      return(resINV_INST);
+
+    // Z280/R800 multuw
+    case 0xc3: // multuw hl, bc
+      if (type->type==CPU_R800)
+        {
+          unsigned long result = (unsigned long)(regs.HL) *  (unsigned long)(regs.BC);
+          regs.HL = (result >> 0) & 0xff;
+          regs.DE = (result >> 16) & 0xff;
+          regs.raf.F &= ~(BIT_S | BIT_Z | BIT_P | BIT_C);
+          if (!result)
+            regs.raf.F |= BIT_Z;
+          if (regs.DE)
+            regs.raf.F |= BIT_C;
+          tick (36);
+          return(resGO);
+        }
+      return(resINV_INST);
+    case 0xf3: // multuw hl, sp
+      //todo
+      return(resINV_INST);
+
     default:
       return(resINV_INST);
     }
