@@ -168,7 +168,7 @@ static bool XAinst_ok(const assignment &a, unsigned short int i, const G_t &G, c
     ic->op == ADDRESS_OF ||
     ic->op == CAST ||
     ic->op == DUMMY_READ_VOLATILE ||
-    ic->op == SWAP)
+    ic->op == ROT && IS_OP_LITERAL (IC_RIGHT (ic)) && (bitsForType (operandType (IC_LEFT (ic))) == 8 || operandLitValueUll (IC_RIGHT (ic)) * 2 == bitsForType (operandType (IC_LEFT (ic)))))
     return(true);
 
   if(ic->op == IFX && ic->generated)
@@ -267,7 +267,7 @@ static bool AXinst_ok(const assignment &a, unsigned short int i, const G_t &G, c
     ic->op == DUMMY_READ_VOLATILE ||
     ic->op == CRITICAL ||
     ic->op == ENDCRITICAL ||
-    ic->op == SWAP)
+    ic->op == ROT && IS_OP_LITERAL (IC_RIGHT (ic)) && (bitsForType (operandType (IC_LEFT (ic))) == 8 || operandLitValueUll (IC_RIGHT (ic)) * 2 == bitsForType (operandType (IC_LEFT (ic)))))
     return(true);
 
   bool unused_A = (ia.registers[REG_A][1] < 0);
@@ -350,16 +350,9 @@ static void assign_operands_for_cost(const assignment &a, unsigned short int i,
 {
   const iCode *ic = G[i].ic;
   
-  if(ic->op == IFX)
-    assign_operand_for_cost(IC_COND(ic), a, i, G, I);
-  else if(ic->op == JUMPTABLE)
-    assign_operand_for_cost(IC_JTCOND(ic), a, i, G, I);
-  else
-    {
-      assign_operand_for_cost(IC_LEFT(ic), a, i, G, I);
-      assign_operand_for_cost(IC_RIGHT(ic), a, i, G, I);
-      assign_operand_for_cost(IC_RESULT(ic), a, i, G, I);
-    }
+  assign_operand_for_cost(IC_LEFT(ic), a, i, G, I);
+  assign_operand_for_cost(IC_RIGHT(ic), a, i, G, I);
+  assign_operand_for_cost(IC_RESULT(ic), a, i, G, I);
     
   if(ic->op == SEND && ic->builtinSEND)
     {
@@ -402,6 +395,8 @@ static bool operand_sane(const operand *o, const assignment &a, unsigned short i
       while(++oi != oi_end)
         if(!std::binary_search(a.local.begin(), a.local.end(), oi->second))
           return(false);
+      if (OP_SYMBOL_CONST (o)->nRegs > 2) // cannot handle register operand wider than 2 B yet.
+        return (false);
     }
   else
     {
@@ -488,6 +483,7 @@ static float instruction_cost(const assignment &a, unsigned short int i, const G
     case GETABIT:
     case GETBYTE:
     case GETWORD:
+    case ROT:
     case LEFT_OP:
     case RIGHT_OP:
     case GET_VALUE_AT_ADDRESS:
@@ -501,7 +497,6 @@ static float instruction_cost(const assignment &a, unsigned short int i, const G
     case DUMMY_READ_VOLATILE:
     case CRITICAL:
     case ENDCRITICAL:
-    case SWAP:
       assign_operands_for_cost(a, i, G, I);
       set_surviving_regs(a, i, G, I);
       c = dryhc08iCode(ic);
@@ -651,6 +646,11 @@ iCode *hc08_ralloc2_cc(ebbIndex *ebbi)
 
   iCode *ic = create_cfg(control_flow_graph, conflict_graph, ebbi);
 
+  if (optimize.genconstprop)
+    recomputeValinfos (ic, ebbi, "_2");
+
+  guessCounts(ic, ebbi);
+
   if(options.dump_graphs)
     dump_cfg(control_flow_graph);
 
