@@ -87,8 +87,11 @@ pdk_genInitStartup (FILE *of)
   fprintf (of, "p::\n");
   fprintf (of, "\t.ds 2\n");
   
+  // WARNING: program memory address 0x20:21 (0x10 in words) is reserved for the
+  // global singular interrupt vector. Ensure startup code never grows too large
+  // and intrudes on this location.
 
-  fprintf (of, "\t.area\tHEADER (ABS)\n"); // In the header we have 16 bytes. First should be nop.
+  fprintf (of, "\t.area\tHEADER (ABS)\n");
   fprintf (of, "\t.org 0x0000\n");
   fprintf (of, "\tnop\n"); // First word is a jump to self-test routine at end of ROM on some new devices.
 
@@ -109,8 +112,22 @@ pdk_genInitStartup (FILE *of)
       fprintf (of, "\tmov.io\tsp, a\n");
     }
 
-  fprintf (of, "\tcall\t__sdcc_external_startup\n");
-  fprintf (of, "\tgoto\ts_GSINIT\n");
+  /* Skip init if __sdcc_external_startup returned non-zero */
+  fprintf (of, "\tcall\t___sdcc_external_startup\n");
+  if (TARGET_IS_PDK13)
+    {
+      // No "cneqsn" instruction on pdk13 - we must invert the logic.
+      fprintf (of, "\tnot\ta\n");
+      fprintf (of, "\tceqsn\ta, #0xFF\n");
+      fprintf (of, "\tgoto\t__sdcc_program_startup\n");
+      fprintf (of, "\tgoto\t__sdcc_init_data\n");
+    }
+  else
+    {
+      fprintf (of, "\tcneqsn\ta, #0x00\n");
+      fprintf (of, "\tgoto\t__sdcc_init_data\n");
+      fprintf (of, "\tgoto\t__sdcc_program_startup\n");
+    }
 
   tfprintf (of, "\t!area\n", STATIC_NAME);
 
@@ -195,14 +212,19 @@ _hasNativeMulFor (iCode *ic, sym_link *left, sym_link *right)
   if (ic->op != '*')
     return (false);
 
+  if (IS_BITINT (OP_SYM_TYPE (IC_RESULT(ic))) && SPEC_BITINTWIDTH (OP_SYM_TYPE (IC_RESULT(ic))) % 8)
+    return false;
+
   return ((IS_LITERAL (left) || IS_LITERAL (right)) && result_size == 1);
 }
 
 /* Indicate which extended bit operations this backend supports */
 static bool
-hasExtBitOp (int op, int size)
+hasExtBitOp (int op, sym_link *left, int right)
 {
-  return (op == GETBYTE || op == SWAP && size == 1);
+  return (op == GETBYTE ||
+    op == ROT && bitsForType (left) == 8 ||
+    op == ROT && !(bitsForType (left) % 8) && right <= 2);
 }
 
 static const char *
@@ -215,11 +237,12 @@ get_model (void)
     $2 is always the output file.
     $3 varies
     $l is the list of extra options that should be there somewhere...
+    $L is the list of extra options that should be passed on the command line...
     MUST be terminated with a NULL.
 */
 static const char *_linkCmd[] =
 {
-  "sdldpdk", "-nf", "\"$1\"", NULL
+  "sdldpdk", "-nf", "\"$1\"", "$L", NULL
 };
 
 /* $3 is replaced by assembler.debug_opts resp. port->assembler.plain_opts */
@@ -234,7 +257,7 @@ PORT pdk13_port =
 {
   TARGET_ID_PDK13,
   "pdk13",
-  "PDK13",                       /* Target name */
+  "Padauk PDK13",                /* Target name */
   0,                             /* Processor name */
   {
     glue,
@@ -287,9 +310,10 @@ PORT pdk13_port =
     0,                          /* banked func ptr */
     1,                          /* bit */
     4,                          /* float */
+    64,                         /* bit-precise integer types up to _BitInt (64) */
   },
   /* tags for generic pointers */
-  { 0x00, 0x40, 0x60, 0x80 },   /* far, near, xstack, code */
+  { 0x00, 0x00, 0x00, 0x80 },   /* far, near, xstack, code */
   {
     "XSEG",
     "STACK",
@@ -315,6 +339,7 @@ PORT pdk13_port =
     0,
     0,
     1,                          /* CODE  is read-only */
+    false,                      // no instructions accessing i/o space with indirect addressing available in hardware
     1                           /* No fancy alignments supported. */
   },
   { 0, 0 },
@@ -328,7 +353,7 @@ PORT pdk13_port =
      2,
      1,                         /* sp points to next free stack location */
   },     
-  { -1, false },                /* no int x int -> long multiplication support routine. */
+  { -1, false, false },         /* Neither int x int -> long nor unsigned long x unsigned char -> unsigned long long multiplication support routine. */
   { pdk_emitDebuggerSymbol,
     {
       0,
@@ -403,7 +428,7 @@ PORT pdk14_port =
 {
   TARGET_ID_PDK14,
   "pdk14",
-  "PDK14",                       /* Target name */
+  "Padauk PDK14",                /* Target name */
   0,                             /* Processor name */
   {
     glue,
@@ -454,9 +479,10 @@ PORT pdk14_port =
     0,                          /* banked func ptr */
     1,                          /* bit */
     4,                          /* float */
+    64,                         /* bit-precise integer types up to _BitInt (64) */
   },
   /* tags for generic pointers */
-  { 0x00, 0x40, 0x60, 0x80 },   /* far, near, xstack, code */
+  { 0x00, 0x00, 0x00, 0x80 },   /* far, near, xstack, code */
   {
     "XSEG",
     "STACK",
@@ -482,6 +508,7 @@ PORT pdk14_port =
     0,
     0,
     1,                          /* CODE  is read-only */
+    false,                      // no instructions accessing i/o space with indirect addressing available in hardware
     1                           /* No fancy alignments supported. */
   },
   { 0, 0 },
@@ -495,7 +522,7 @@ PORT pdk14_port =
      2,
      1,                         /* sp points to next free stack location */
   },     
-  { -1, false },                /* no int x int -> long multiplication support routine. */
+  { -1, false, false },         /* Neither int x int -> long nor unsigned long x unsigned char -> unsigned long long multiplication support routine. */
   { pdk_emitDebuggerSymbol,
     {
       0,
@@ -570,7 +597,7 @@ PORT pdk15_port =
 {
   TARGET_ID_PDK15,
   "pdk15",
-  "PDK15",                       /* Target name */
+  "Padauk PDK15",                /* Target name */
   0,                             /* Processor name */
   {
     glue,
@@ -621,9 +648,10 @@ PORT pdk15_port =
     0,                          /* banked func ptr */
     1,                          /* bit */
     4,                          /* float */
+    64,                         /* bit-precise integer types up to _BitInt (64) */
   },
   /* tags for generic pointers */
-  { 0x00, 0x40, 0x60, 0x80 },   /* far, near, xstack, code */
+  { 0x00, 0x00, 0x00, 0x80 },   /* far, near, xstack, code */
   {
     "XSEG",
     "STACK",
@@ -649,6 +677,7 @@ PORT pdk15_port =
     0,
     0,
     1,                          /* CODE  is read-only */
+    false,                      // no instructions accessing i/o space with indirect addressing available in hardware
     1                           /* No fancy alignments supported. */
   },
   { 0, 0 },
@@ -662,7 +691,7 @@ PORT pdk15_port =
      2,
      1,                         /* sp points to next free stack location */
   },     
-  { -1, false },                /* no int x int -> long multiplication support routine. */
+  { -1, false, false },         /* Neither int x int -> long nor unsigned long x unsigned char -> unsigned long long multiplication support routine. */
   { pdk_emitDebuggerSymbol,
     {
       0,
