@@ -349,8 +349,25 @@ packRegsForOneuse (iCode *ic, operand **opp, eBBlock *ebp)
     || !isOperandGlobal (IC_RIGHT (dic)))
     return 0;
 
-  if (IS_OP_VOLATILE (IC_RESULT (ic)) && IS_OP_VOLATILE (IC_RIGHT (dic)))
-    return 0;
+  if (IS_OP_VOLATILE (IC_RESULT (ic)) && IS_OP_VOLATILE (IC_RIGHT (dic))) // Only case with two volatiles that we can optimize: Some bitwise operation on __sfr.
+    {
+      if (ic->op != BITWISEAND && ic->op != '|' && ic->op != '^' || operandSize (IC_LEFT (ic)) != 1 || !IS_OP_LITERAL (IC_RIGHT (ic)))
+        return 0;
+
+      if (!OP_SYMBOL (IC_RESULT (ic)) || !IN_REGSP (SPEC_OCLS (OP_SYMBOL (IC_RESULT (ic))->etype)))
+        return 0;
+  
+      if (OP_SYMBOL (IC_RIGHT (dic))->key != OP_SYMBOL (IC_RESULT (ic))->key)
+        return 0;
+
+      unsigned int val = operandLitValueUll (IC_RIGHT (ic));
+      if (ic->op == BITWISEAND)
+        val = ~val;
+      val &= 0xff;
+
+      if(ic->op != '^' && val != 0x01 && val != 0x02 && val != 0x04 && val != 0x08 && val != 0x10 && val != 0x20 && val != 0x40 && val != 0x80)
+        return 0;
+    }
 
   /* also make sure the intervenening instructions
      don't have any thing in far space */
@@ -454,7 +471,7 @@ packRegisters (eBBlock * ebp)
          cast is remat, then we can remat this cast as well */
       if (ic->op == CAST &&
         IS_SYMOP (IC_RIGHT (ic)) && OP_SYMBOL (IC_RIGHT (ic))->remat &&
-        !isOperandGlobal (IC_RESULT (ic)) && bitVectnBitsOn (OP_DEFS (IC_RESULT (ic))) == 1 && !IS_PARM (IC_RESULT (ic)) && /* The receiving of the paramter is not accounted for in DEFS */
+        !isOperandGlobal (IC_RESULT (ic)) && bitVectnBitsOn (OP_DEFS (IC_RESULT (ic))) == 1 && !IS_PARM (IC_RESULT (ic)) && /* The receiving of the parameter is not accounted for in DEFS */
         !OP_SYMBOL (IC_RESULT (ic))->addrtaken)
         {
           sym_link *to_type = operandType (IC_LEFT (ic));
@@ -506,24 +523,30 @@ packRegisters (eBBlock * ebp)
           operand *op = IC_RESULT (ic);
           if ((use->op == LEFT_OP || use->op == '+' || use->op == '-' || use->op == UNARYMINUS ||
             use->op == '&' || use->op == '|' || use->op == '^') &&
-            IC_LEFT (use)->key == op->key && (!IC_RIGHT(use) || IC_RIGHT (use)->key != op->key))
+            IC_LEFT (use)->key == op->key && (!IC_RIGHT (use) || IC_RIGHT (use)->key != op->key))
             {
-              bitVectUnSetBit (OP_SYMBOL (IC_RIGHT (ic))->uses, ic->key);
-              bitVectSetBit (OP_SYMBOL (IC_RIGHT (ic))->uses, use->key);
+              if (IS_SYMOP (ic->right))
+                {
+                  bitVectUnSetBit (OP_SYMBOL (ic->right)->uses, ic->key);
+                  bitVectSetBit (OP_SYMBOL (ic->right)->uses, use->key);
+                }
               IC_LEFT (use) = operandFromOperand (IC_RIGHT(ic));
               remiCodeFromeBBlock (ebp, ic);
               hTabDeleteItem (&iCodehTab, ic->key, ic, DELETE_ITEM, NULL);
               if(ic->prev)
                 ic = ic->prev;
             }
-          else if ((/*(use->op == SET_VALUE_AT_ADDRESS && !IS_BITVAR (getSpec (operandType (IC_LEFT (use)))) && !IS_BITVAR (getSpec (operandType (IC_RIGHT (use))))) || - resulted in pointer writes toring too few bytes*/
+          else if ((/*(use->op == SET_VALUE_AT_ADDRESS && !IS_BITVAR (getSpec (operandType (IC_LEFT (use)))) && !IS_BITVAR (getSpec (operandType (IC_RIGHT (use))))) || - resulted in pointer write storing too few bytes*/
             use->op == CAST && (SPEC_USIGN (getSpec (operandType (IC_RIGHT (use)))) || operandSize (IC_RESULT (use)) <= operandSize (IC_RIGHT (use))) ||
             use->op == LEFT_OP || use->op == RIGHT_OP || use->op == '+' || use->op == '-' ||
             use->op == '&' || use->op == '|' || use->op == '^') &&
             IC_RIGHT (use)->key == op->key && (!IC_LEFT(use) || IC_LEFT (use)->key != op->key))
             {
-              bitVectUnSetBit (OP_SYMBOL (IC_RIGHT (ic))->uses, ic->key);
-              bitVectSetBit (OP_SYMBOL (IC_RIGHT (ic))->uses, use->key);
+              if (IS_SYMOP (ic->right))
+                {
+                  bitVectUnSetBit (OP_SYMBOL (IC_RIGHT (ic))->uses, ic->key);
+                  bitVectSetBit (OP_SYMBOL (IC_RIGHT (ic))->uses, use->key);
+                }
               IC_RIGHT (use) = operandFromOperand (IC_RIGHT(ic));
               remiCodeFromeBBlock (ebp, ic);
               hTabDeleteItem (&iCodehTab, ic->key, ic, DELETE_ITEM, NULL);
@@ -556,16 +579,13 @@ serialRegMark (eBBlock **ebbs, int count)
       /* for all instructions do */
       for (ic = ebbs[i]->sch; ic; ic = ic->next)
         {
-          if (ic->op == IPOP)
-            wassert (0);
-
           /* if result is present && is a true symbol */
           if (IC_RESULT (ic) && ic->op != IFX && IS_TRUE_SYMOP (IC_RESULT (ic)))
             OP_SYMBOL (IC_RESULT (ic))->allocreq++;
 
           /* some don't need registers, since there is no result. */
           if (SKIP_IC2 (ic) ||
-              ic->op == JUMPTABLE || ic->op == IFX || ic->op == IPUSH || ic->op == IPOP || ic->op == SET_VALUE_AT_ADDRESS)
+              ic->op == JUMPTABLE || ic->op == IFX || ic->op == IPUSH || ic->op == SET_VALUE_AT_ADDRESS)
             continue;
 
           /* now we need to allocate registers only for the result */
@@ -581,7 +601,7 @@ serialRegMark (eBBlock **ebbs, int count)
                   sym->isspilt = FALSE;
                 }
 
-              /* Make sure any spill location is definately allocated */
+              /* Make sure any spill location is definitely allocated */
               if (sym->isspilt && !sym->remat && sym->usl.spillLoc && !sym->usl.spillLoc->allocreq)
                 sym->usl.spillLoc->allocreq++;
 
@@ -601,7 +621,7 @@ serialRegMark (eBBlock **ebbs, int count)
                   sym->isspilt = false;
                 }
 
-              if (sym->nRegs > 2 && ic->op == CALL) // To be allocated to stack due to the way (long) long return values are handled via a hidden pointer.
+              if (sym->nRegs > 2 && (ic->op == CALL || ic->op == PCALL)) // To be allocated to stack due to the way (long) long return values are handled via a hidden pointer.
                 {
                   sym->for_newralloc = 0;
                   pdkSpillThis (sym);
@@ -677,18 +697,6 @@ pdkRegFix (eBBlock ** ebbs, int count)
           if (SKIP_IC2 (ic))
             continue;
 
-          if (ic->op == IFX)
-            {
-              verifyRegsAssigned (IC_COND (ic), ic);
-              continue;
-            }
-
-          if (ic->op == JUMPTABLE)
-            {
-              verifyRegsAssigned (IC_JTCOND (ic), ic);
-              continue;
-            }
-
           verifyRegsAssigned (IC_RESULT (ic), ic);
           verifyRegsAssigned (IC_LEFT (ic), ic);
           verifyRegsAssigned (IC_RIGHT (ic), ic);
@@ -706,8 +714,6 @@ pdk_assignRegisters (ebbIndex *ebbi)
   int count = ebbi->count;
   iCode *ic;
 
-  pdk_init_asmops();
-
   transformPointerSet (ebbs, count);
 
   /* change assignments this will remove some
