@@ -316,16 +316,9 @@ static void assign_operands_for_cost(const assignment &a, unsigned short int i,
 {
   const iCode *ic = G[i].ic;
   
-  if(ic->op == IFX)
-    assign_operand_for_cost(IC_COND(ic), a, i, G, I);
-  else if(ic->op == JUMPTABLE)
-    assign_operand_for_cost(IC_JTCOND(ic), a, i, G, I);
-  else
-    {
-      assign_operand_for_cost(IC_LEFT(ic), a, i, G, I);
-      assign_operand_for_cost(IC_RIGHT(ic), a, i, G, I);
-      assign_operand_for_cost(IC_RESULT(ic), a, i, G, I);
-    }
+  assign_operand_for_cost(IC_LEFT(ic), a, i, G, I);
+  assign_operand_for_cost(IC_RIGHT(ic), a, i, G, I);
+  assign_operand_for_cost(IC_RESULT(ic), a, i, G, I);
     
   if(ic->op == SEND && ic->builtinSEND)
     assign_operands_for_cost(a, (unsigned short)*(adjacent_vertices(i, G).first), G, I);
@@ -424,7 +417,7 @@ static float instruction_cost(const assignment &a, unsigned short int i, const G
     case '|':
     case BITWISEAND:
     case IPUSH:
-    //case IPOP:
+    case IPUSH_VALUE_AT_ADDRESS:
     case CALL:
     case PCALL:
     case RETURN:
@@ -442,6 +435,7 @@ static float instruction_cost(const assignment &a, unsigned short int i, const G
     //case GETABIT:
     //case GETBYTE:
     //case GETWORD:
+    case ROT:
     case LEFT_OP:
     case RIGHT_OP:
     case GET_VALUE_AT_ADDRESS:
@@ -456,7 +450,6 @@ static float instruction_cost(const assignment &a, unsigned short int i, const G
     case DUMMY_READ_VOLATILE:
     /*case CRITICAL:
     case ENDCRITICAL:*/
-    case SWAP:
       assign_operands_for_cost(a, i, G, I);
       set_surviving_regs(a, i, G, I);
       c = dryPdkiCode(ic);
@@ -658,6 +651,11 @@ iCode *pdk_ralloc2_cc(ebbIndex *ebbi)
 
   ic = create_cfg(control_flow_graph, conflict_graph, ebbi);
 
+  if(optimize.genconstprop)
+    recomputeValinfos(ic, ebbi, "_2");
+
+  guessCounts(ic, ebbi);
+
   if(options.dump_graphs)
     dump_cfg(control_flow_graph);
 
