@@ -78,6 +78,7 @@ typedef struct asmop
     asmop_byte bytes[8];
   } aopu;
   signed char regs[6]; // Byte of this aop that is in the register. -1 if no byte of this aop is in the reg.
+  struct valinfo valinfo;
 }
 asmop;
 
@@ -87,7 +88,7 @@ void stm8_emitDebuggerSymbol (const char *);
 bool stm8IsReturned(const char *what);
 
 // Check if what is part of the ith argument (counting from 1) to a function of type ftype.
-// If what is 0, just check if hte ith argument is in registers.
+// If what is 0, just check if the ith argument is in registers.
 bool stm8IsRegArg(struct sym_link *ftype, int i, const char *what);
 
 // Check if what is part of the any argument (counting from 1) to a function of type ftype.
