@@ -3,19 +3,17 @@
 #include "SDCCglobl.h"
 #include "SDCCgen.h"
 
+#include "peep.h"
+#include "gen.h"
+
 #define NOTUSEDERROR() do {werror(E_INTERNAL_ERROR, __FILE__, __LINE__, "error in notUsed()");} while(0)
 
-//#define D(_s) { printf _s; fflush(stdout); }
+// #define D(_s) { printf _s; fflush(stdout); }
 #define D(_s)
 
-#define EQUALS(l, i) (!strcmp((l), (i)))
-#define ISINST(l, i) (!strncmp((l), (i), sizeof(i) - 1))
-
-// This function should behave just like C99 isblank(). Remove it, once MSVC supports isblank().
-static int _isblank(int c)
-{
-  return (c == ' ' || c == '\t');
-}
+#define EQUALS(l, i) (!STRCASECMP((l), (i)))
+#define ISINST(l, i) (!STRNCASECMP((l), (i), sizeof(i) - 1) && (!(l)[sizeof(i) - 1] || isspace((unsigned char)((l)[sizeof(i) - 1]))))
+#define STARTSINST(l, i) (!STRNCASECMP((l), (i), sizeof(i) - 1))
 
 typedef enum
 {
@@ -33,17 +31,67 @@ static struct
   lineNode *head;
 } _G;
 
+extern bool stm8_regs_used_as_parms_in_calls_from_current_function[YH_IDX + 1];
+extern bool stm8_regs_used_as_parms_in_pcalls_from_current_function[YH_IDX + 1];
+
+/*----------------------------------------------------------------------------*/
+/* strNextCharBlock - Returns the next block of chars (after spaces, comma)   */
+/* Leading spaces and Current block are skipped and search stops at next block*/
+/* Valid block separators are: ' ' and ','                                    */
+/* If no block is found (EOS or ';'), returns NULL                            */
+/*----------------------------------------------------------------------------*/
+static char *
+strNextCharBlock(const char *str)
+{
+  if (!str || !str[0])
+    return 0;
+
+  while (isblank ((unsigned char)(str[0])))
+    str++; // skip leading blanks
+
+  while (str[0] && !isblank ((unsigned char)(str[0])) && str[0] != ';')
+    {
+      if (str[0] == ',')
+        {
+          str++; // current block is finished with ','
+          break;
+        }
+      str++; // next char of current block
+    }
+
+  while (isblank ((unsigned char)(str[0])))
+    str++; // skip trailing blanks 
+    
+  if (str[0] && str[0] != ';')
+    return (char *)str;
+  return 0;
+}
+
+static bool
+isInt(const char *str)
+{
+  int ret;
+  while(str[0] == '#' || str[0] == '(' || str[0] == '[' || isspace ((unsigned char)str[0]))
+    str++;
+  if(sscanf(str, "0x%x", &ret))
+    return(ret);
+  if(!sscanf(str, "%d", &ret))
+    return(false);
+  return(true);
+}
+
 static int
 readint(const char *str)
 {
   int ret;
-  while(str[0] == '#' || str[0] == '(')
+  while(str[0] == '#' || str[0] == '(' || str[0] == '[' || isspace ((unsigned char)str[0]))
     str++;
   if(sscanf(str, "0x%x", &ret))
     return(ret);
   if(!sscanf(str, "%d", &ret))
     {
-      wassertl (0, "readint() got non-integer argument.");
+      wassertl (0, "readint() got non-integer argument:");
+      fprintf (stderr, "%s\n", str);
       ret = -1;
     }
   return(ret);
@@ -55,10 +103,10 @@ isReg(const char *what)
   if(what[0] == '(')
     what++;
   if(what[0] == 'a' || what[0] == 'x' || what[0] == 'y')
-    return(TRUE);
+    return(true);
   if(!strcmp(what, "sp"))
-    return(TRUE);
-  return(FALSE);
+    return(true);
+  return(false);
 }
 
 static char *
@@ -96,7 +144,7 @@ nextToken(char *p)
   return(ret);
 }
 
-static int
+static bool
 isRelativeAddr(const char *what, const char *mode)
 {
   char buf[4];
@@ -105,45 +153,124 @@ isRelativeAddr(const char *what, const char *mode)
   return(what[0] == '(' && strstr(what, buf));
 }
 
-static int
+static bool
 isLabel(const char *what)
 {
-  const char *end;    
+  const char *end;
+
   end = strchr(what, '+');
   if(!end)
     end = what + strlen(what);
+  if(what[0] == '(' && !strchr(what, ','))
+    what++;
   if(what[0] == '#')
     return (what[1] == '_' || what[1] == '<' || what[1] == '>');
   return(what[0] == '_' || *(end-1) == '$');
 }
 
-static int
+static bool
 isImmediate(const char *what)
 {
   return(what[0] == '#');
 }
 
-static int
+static bool
 isShortoff(const char *what, const char *mode)
 {
-  return(isRelativeAddr(what, mode) && readint(what) <= 0xFF);
+  return(isRelativeAddr(what, mode) && isInt(what) && readint(what) <= 0xff);
 }
 
-static int
+static bool
 isLongoff(const char *what, const char *mode)
 {
-  return(isRelativeAddr(what, mode) && readint(what) > 0xFF);
+  return(isRelativeAddr(what, mode) && (!isInt(what) || readint(what) > 0xff));
 }
 
-static int
+static bool
+isPtr(const char *what)
+{
+  return(what[0] == '[' || what[0] == '(' && (what[1] == '[' || what[1] == '('));
+}
+
+static bool
 isSpIndexed(const char *what)
 {
   return isRelativeAddr(what, "sp");
 }
 
+/*-----------------------------------------------------------------*/
+/* stm8InstIsRegToReg - Checks if 'line' is a reg to reg move      */
+/* isword == FALSE : Look for registers a, xl, xh, yl & yh         */
+/* isword == TRUE  : Look for registers x, y & sp                  */
+/*-----------------------------------------------------------------*/
+static bool
+stm8InstIsRegToReg(const char *line, bool isword)
+{
+  int regNumber = 0;
+
+  if ((line = strNextCharBlock(line)))
+    {
+      while(line[0])
+        {
+          bool regFound = false;
+          char chrLow = tolower ((unsigned char)line[0]);
+
+          // Check for register names
+          if (isword)
+            {
+              if (chrLow == 'x' || chrLow == 'y')
+                regFound = true;
+
+              if (chrLow == 's')
+                {
+                  line++;
+                  if (line[0] == 'p')
+                    regFound = true;
+                  else
+                    return false;
+                }
+            }
+          else
+            {
+              if (chrLow == 'a')
+                regFound = true;
+
+              if (chrLow == 'x' || chrLow == 'y')
+                {
+                  line++;
+                  chrLow = tolower ((unsigned char)line[0]);
+                  if (chrLow == 'h' || chrLow == 'l')
+                    regFound = true;
+                  else
+                    return false;
+                }
+            }
+
+          // If register, process next character
+          if (regFound)
+            line++;
+
+          // Continue only if valid separator or end
+          if (!line[0] || line[0] == ',' || isblank (line[0]))
+            {
+              if (regFound)
+                regNumber++;
+            }
+          else
+            {
+              return false;
+            }
+
+          // Next char if not eos
+          if(line[0])
+            line++;
+        }
+    }
+  return (regNumber == 2);
+}
 
 int
-stm8instructionSize(const lineNode *pl)
+stm8instructionSize(lineNode *pl)
 { // this function is quite rough, it makes all indirect addressing cases to the longest.
   char *operand;
   char *op1start;
@@ -153,63 +280,63 @@ stm8instructionSize(const lineNode *pl)
   op1start = nextToken(NULL);
   op2start = nextToken(NULL);
 
-  while(op1start && isspace(*op1start)) op1start++;
-  while(op2start && isspace(*op2start)) op2start++;
+  while(op1start && isspace((unsigned char)op1start[0])) op1start++;
+  while(op2start && isspace((unsigned char)op2start[0])) op2start++;
   //printf("line=%s operand=%s op1start=%s op2start=%s\n", pl->line, operand, op1start, op2start);
 
   /* Operations that always costs 1 byte */
-  if (EQUALS(operand, "ccf")
-    || EQUALS(operand, "divw")
-    || EQUALS(operand, "exgw")
-    || EQUALS(operand, "iret")
-    || EQUALS(operand, "nop")
-    || EQUALS(operand, "rcf")
-    || EQUALS(operand, "ret")
-    || EQUALS(operand, "retf")
-    || EQUALS(operand, "rvf")
-    || EQUALS(operand, "break")
-    || EQUALS(operand, "halt")
-    || EQUALS(operand, "rim")
-    || EQUALS(operand, "trap")
-    || EQUALS(operand, "wfi")
-    || EQUALS(operand, "sim")
-    || EQUALS(operand, "scf"))
+  if (ISINST(operand, "ccf")
+    || ISINST(operand, "divw")
+    || ISINST(operand, "exgw")
+    || ISINST(operand, "iret")
+    || ISINST(operand, "nop")
+    || ISINST(operand, "rcf")
+    || ISINST(operand, "ret")
+    || ISINST(operand, "retf")
+    || ISINST(operand, "rvf")
+    || ISINST(operand, "break")
+    || ISINST(operand, "halt")
+    || ISINST(operand, "rim")
+    || ISINST(operand, "trap")
+    || ISINST(operand, "wfi")
+    || ISINST(operand, "sim")
+    || ISINST(operand, "scf"))
       return 1;
 
   /* Operations that always costs 3 byte */
-  if(EQUALS(operand, "jrh")
-    || EQUALS(operand, "jrnh")
-    || EQUALS(operand, "jril")
-    || EQUALS(operand, "jrih")
-    || EQUALS(operand, "jrm")
-    || EQUALS(operand, "jrnm"))
+  if(ISINST(operand, "jrh")
+    || ISINST(operand, "jrnh")
+    || ISINST(operand, "jril")
+    || ISINST(operand, "jrih")
+    || ISINST(operand, "jrm")
+    || ISINST(operand, "jrnm"))
       return 3;
 
   /* Operations that always costs 2 byte */
-  if(ISINST(operand, "jr")
-    || EQUALS(operand, "callr")
-    || EQUALS(operand, "wfe"))
+  if(STARTSINST(operand, "jr")
+    || ISINST(operand, "callr")
+    || ISINST(operand, "wfe"))
       return 2;
 
   /* Operations that always costs 4 byte */
-  if(EQUALS(operand, "bccm")
-    || EQUALS(operand, "bcpl")
-    || EQUALS(operand, "bres")
-    || EQUALS(operand, "bset")
-    || EQUALS(operand, "callf")
-    || EQUALS(operand, "int")
-    || EQUALS(operand, "jpf"))
+  if(ISINST(operand, "bccm")
+    || ISINST(operand, "bcpl")
+    || ISINST(operand, "bres")
+    || ISINST(operand, "bset")
+    || ISINST(operand, "callf")
+    || ISINST(operand, "int")
+    || ISINST(operand, "jpf"))
       return 4;
 
   /* Operations that always costs 5 byte */
-  if(EQUALS(operand, "btjf")
-    || EQUALS(operand, "btjt"))
+  if(ISINST(operand, "btjf")
+    || ISINST(operand, "btjt"))
       return 5;
 
   if (EQUALS(operand, "push")
     || EQUALS(operand, "pop"))
   {
-    assert (op1start != NULL);
+    wassert (op1start);
     if (!strcmp(op1start, "a"))
       return 1;
     if (!strcmp(op1start, "cc"))
@@ -239,7 +366,7 @@ stm8instructionSize(const lineNode *pl)
   {
     int i = 0;
 
-    assert (op1start != NULL);
+    wassert (op1start);
     if(!strcmp(op1start, "a") || !strcmp(op1start, "(x)"))
       return(1);
     if(!strcmp(op1start, "(y)"))
@@ -248,12 +375,14 @@ stm8instructionSize(const lineNode *pl)
       op1start++;
     if(strstr(op1start, ",y)"))
       i++; // costs extra byte for operating with y
+    if ((ISINST(operand, "jp") || ISINST(operand, "call")) && *op1start != '(' && *op1start != '[') // jp and call are 3 bytes for direct long addressing mode.
+      return(3);
     if(isLabel(op1start))
       return(4);
     if(readint(op1start) <= 0xFF)
       return(2+i);
     /* op1 > 0xFF */
-    if((EQUALS(operand, "jp") || EQUALS(operand, "call")) && !strchr(op1start, 'y'))
+    if((ISINST(operand, "jp") || ISINST(operand, "call")) && !strchr(op1start, 'y'))
       return(3);
     return(4);
   }
@@ -281,7 +410,7 @@ stm8instructionSize(const lineNode *pl)
     return(4);
   }
 
-  if(EQUALS(operand, "cplw"))
+  if(ISINST(operand, "cplw"))
   {
     assert (op1start != NULL);
     if(op1start[0] == 'y')
@@ -290,7 +419,7 @@ stm8instructionSize(const lineNode *pl)
       return(1);
   }
 
-  if(EQUALS(operand, "ldf"))
+  if(ISINST(operand, "ldf"))
   {
     assert (op1start != NULL);
     if(isRelativeAddr(op1start, "y") || isRelativeAddr(op2start, "y"))
@@ -300,8 +429,8 @@ stm8instructionSize(const lineNode *pl)
   }
 
   /* Operations that costs 2 or 3 bytes for immediate */
-  if(ISINST(operand, "ld")
-                || ISINST(operand, "cp")
+  if(STARTSINST(operand, "ld")
+                || !strncmp(operand, "cp", 2)
                 || EQUALS(operand, "adc")
                 || EQUALS(operand, "add")
                 || EQUALS(operand, "and")
@@ -313,7 +442,7 @@ stm8instructionSize(const lineNode *pl)
   {
     int i = 0;
     char suffix;
-    assert (op1start != NULL && op2start != NULL);
+    wassert (op1start && op2start);
     suffix = operand[strlen(operand)-1];
     if(suffix == 'w' && isImmediate(op2start))
       {
@@ -337,12 +466,19 @@ stm8instructionSize(const lineNode *pl)
       return(3);
     if(isLongoff(op1start, "y") || isLongoff(op2start, "y"))
       return(4);
+    if(isPtr(op1start) || isPtr(op2start))
+      return(4);
     if(strchr(op1start, 'y') || strchr(op2start, 'y'))
       i++; // costs extra byte for operating with y
     if(isLabel(op1start) || isLabel(op2start))
       return(3+i);
     if(isReg(op1start) && isReg(op2start))
-      return(1+i);
+      {
+        if (!strncmp(op1start, "x", 1) && (!strncmp(op2start, "y", 1) || !strncmp(op2start, "sp", 2))
+          || !strncmp(op1start, "sp", 2) && !strncmp(op2start, "x", 1))
+          return(1);
+        return(1+i);
+      }
     if(!strcmp(op2start, "a"))
       return(3);
     if(readint(op2start) <= 0xFF)
@@ -353,7 +489,7 @@ stm8instructionSize(const lineNode *pl)
   }
 
   /* mov costs 3, 4 or 5 bytes depending on its addressing mode */
-  if(EQUALS(operand, "mov")) {
+  if(ISINST(operand, "mov")) {
     assert (op1start != NULL && op2start != NULL);
     if(isImmediate(op2start))
       return(4);
@@ -393,6 +529,20 @@ stm8instructionSize(const lineNode *pl)
       return(1);
   }
 
+  if(ISINST(pl->line, ".db") || ISINST(pl->line, ".byte"))
+    {
+      int i, j;
+      for(i = 1, j = 0; pl->line[j]; i += pl->line[j] == ',', j++);
+      return(i);
+    }
+
+  if(ISINST(pl->line, ".dw") || ISINST(pl->line, ".word"))
+    {
+      int i, j;
+      for(i = 1, j = 0; pl->line[j]; i += pl->line[j] == ',', j++);
+      return(i * 2);
+    }
+
   return(5); // Maximum instruction size, e.g. btjt.
 }
 
@@ -426,12 +576,16 @@ findLabel (const lineNode *pl)
 
   /* 1. extract label in opcode */
 
-  /* In each mcs51 jumping opcode the label is at the end of the opcode */
+  /* In each jump the label is at the end */
   p = strlen (pl->line) - 1 + pl->line;
 
-  /* scan backward until ',' or '\t' */
+  /* Skip trailing whitespace */
+  while(isspace(*p))
+    p--;
+
+  /* scan backward until space or ',' */
   for (; p > pl->line; p--)
-    if (*p == ',' || *p == '\t')
+    if (isspace(*p) || *p == ',')
       break;
 
   /* sanity check */
@@ -450,13 +604,11 @@ findLabel (const lineNode *pl)
 
   /* 3. search lineNode with label definition and return it */
   for (cpl = _G.head; cpl; cpl = cpl->next)
-    {
-      if (   cpl->isLabel
-          && strncmp (p, cpl->line, strlen(p)) == 0)
-        {
-          return cpl;
-        }
-    }
+    if (cpl->isLabel &&
+      strncmp (p, cpl->line, strlen(p)) == 0 &&
+      cpl->line[strlen(p)] == ':')
+        return cpl;
+
   return NULL;
 }
 
@@ -466,98 +618,94 @@ static bool argCont(const char *arg, char what)
   if (arg == NULL || strlen (arg) == 0 || !(what == 'a' || what == 'x' || what == 'y'))
     return FALSE;
 
-  while (_isblank ((unsigned char)(arg[0])))
+  while (isblank ((unsigned char)(arg[0])))
     arg++;
 
   if (arg[0] == ',')
     arg++;
 
-  while (_isblank ((unsigned char)(arg[0])))
+  while (isblank ((unsigned char)(arg[0])))
     arg++;
 
   if (arg[0] == '#')
     return FALSE;
 
-  if (arg[0] == '(' && arg[1] == '0' && (tolower(arg[2])) == 'x') 
-    arg += 3; // Skip hex prefix to avoid false x positive.
+  if (arg[0] == '(') 
+    arg++;
+  if (arg[0] == '0' && (tolower(arg[1])) == 'x') 
+    arg += 2; // Skip hex prefix to avoid false x positive.
 
   if (strlen(arg) == 0)
     return FALSE;
 
+  if (arg[0] == '_' && what == 'a') // The STM8 has no a-relative addressing modes.
+    return FALSE;
+
   return (strchr(arg, what) != NULL);
 }
 
-static bool
-isReturned(const char *what)
-{
-  symbol *sym;
-  sym_link *sym_lnk;
-  int size;
-  lineNode *l;
-
-  l = _G.head;
-  do
-  {
-    l = l->next;
-  } while(l->isComment || l->ic == NULL || l->ic->op != FUNCTION);
-
-  sym = OP_SYMBOL(IC_LEFT(l->ic));
-
-  if(sym && IS_DECL(sym->type))
-    {
-      // Find size of return value.
-      specifier *spec;
-      if(sym->type->select.d.dcl_type != FUNCTION)
-        NOTUSEDERROR();
-      spec = &(sym->etype->select.s);
-      if(spec->noun == V_VOID)
-        size = 0;
-      else if(spec->noun == V_CHAR || spec->noun == V_BOOL)
-        size = 1;
-      else if(spec->noun == V_INT && !(spec->b_long))
-        size = 2;
-      else
-        size = 4;
-
-      // Check for returned pointer.
-      sym_lnk = sym->type;
-      while (sym_lnk && !IS_PTR (sym_lnk))
-        sym_lnk = sym_lnk->next;
-      if(IS_PTR(sym_lnk))
-        size = 2;
-    }
-  else
-    {
-      NOTUSEDERROR();
-      return TRUE;
-    }
-
-  switch(*what)
-    {
-    case 'a':
-      return(size == 1);
-    case 'x':
-      return(size > 1);
-    case 'y':
-      return(size > 2);
-    default:
-      return FALSE;
-    }
-}
-
 static bool
 stm8MightReadFlag(const lineNode *pl, const char *what)
 {
+  if (strcmp (what, "c") && strcmp (what, "n") && strcmp (what, "z"))
+    return true;
+
   if (ISINST (pl->line, "push"))
      return (pl->line[5] == 'c');
 
+  if (!strcmp (what, "v"))
+    return (ISINST (pl->line, "jrnv") || ISINST (pl->line, "jrsge") || ISINST (pl->line, "jrsgt") || ISINST (pl->line, "jrsle") || ISINST (pl->line, "jrslt") || ISINST (pl->line, "jrv"));
+
   if (!strcmp (what, "n"))
-    return (ISINST (pl->line, "jrmi") || ISINST (pl->line, "jrpl") || ISINST (pl->line, "jrsge") || ISINST (pl->line, "jrsgte") || ISINST (pl->line, "jrsle") || ISINST (pl->line, "jrslt"));
+    return (ISINST (pl->line, "jrmi") || ISINST (pl->line, "jrpl") || ISINST (pl->line, "jrsge") || ISINST (pl->line, "jrsgt") || ISINST (pl->line, "jrsle") || ISINST (pl->line, "jrslt"));
 
   if (!strcmp (what, "z"))
-    return (ISINST (pl->line, "jreq") || ISINST (pl->line, "jrne") || ISINST (pl->line, "jrsgte") || ISINST (pl->line, "jrsle"));
+    return (ISINST (pl->line, "jreq") || ISINST (pl->line, "jrne") || ISINST (pl->line, "jrsgt") || ISINST (pl->line, "jrsle"));
 
-  return TRUE;
+  if (!strcmp (what, "c"))
+    return (ISINST (pl->line, "jrc") || ISINST (pl->line, "jrnc") || ISINST (pl->line, "jruge") || ISINST (pl->line, "jrugt") || ISINST (pl->line, "jrule") || ISINST (pl->line, "jrult") ||
+      ISINST (pl->line, "adc") || ISINST (pl->line, "sbc") ||
+      ISINST (pl->line, "ccf") || ISINST (pl->line, "rlc") || ISINST (pl->line, "rlcw") || ISINST (pl->line, "rrc") || ISINST (pl->line, "rrcw"));
+
+  return true;
+}
+
+static bool
+stm8MightBeParmInCallFromCurrentFunction(const char *what)
+{
+  if (!strcmp(what, "a"))
+    return stm8_regs_used_as_parms_in_calls_from_current_function[A_IDX];
+    
+  if ((!strcmp(what, "x") || !strcmp(what, "xl")) && stm8_regs_used_as_parms_in_calls_from_current_function[XL_IDX])
+    return true;
+  if ((!strcmp(what, "x") || !strcmp(what, "xh")) && stm8_regs_used_as_parms_in_calls_from_current_function[XH_IDX])
+    return true;
+
+  if ((!strcmp(what, "y") || !strcmp(what, "yl")) && stm8_regs_used_as_parms_in_calls_from_current_function[YL_IDX])
+    return true;
+  if ((!strcmp(what, "yx") || !strcmp(what, "yh")) && stm8_regs_used_as_parms_in_calls_from_current_function[YH_IDX])
+    return true;
+
+  return false;
+}
+
+static bool
+stm8MightBeParmInPCallFromCurrentFunction(const char *what)
+{
+  if (!strcmp(what, "a"))
+    return stm8_regs_used_as_parms_in_pcalls_from_current_function[A_IDX];
+
+  if ((!strcmp(what, "x") || !strcmp(what, "xl")) && stm8_regs_used_as_parms_in_pcalls_from_current_function[XL_IDX])
+    return true;
+  if ((!strcmp(what, "x") || !strcmp(what, "xh")) && stm8_regs_used_as_parms_in_pcalls_from_current_function[XH_IDX])
+    return true;
+
+  if ((!strcmp(what, "y") || !strcmp(what, "yl")) && stm8_regs_used_as_parms_in_pcalls_from_current_function[YL_IDX])
+    return true;
+  if ((!strcmp(what, "yx") || !strcmp(what, "yh")) && stm8_regs_used_as_parms_in_pcalls_from_current_function[YH_IDX])
+    return true;
+
+  return false;
 }
 
 static bool
@@ -576,7 +724,7 @@ stm8MightRead(const lineNode *pl, const char *what)
     {
       if (ISINST (pl->line, "adc")
         || ISINST (pl->line, "and")
-        || ISINST (pl->line, "bcp") && !ISINST (pl->line, "bcpl")
+        || ISINST (pl->line, "bcp")
         || ISINST (pl->line, "cp")
         || ISINST (pl->line, "div")
         || ISINST (pl->line, "mul")
@@ -588,8 +736,7 @@ stm8MightRead(const lineNode *pl, const char *what)
         || ISINST (pl->line, "xor"))
           return TRUE;
 
-      if (pl->line[4] == 'a' &&
-        (ISINST (pl->line, "add")
+      if ((ISINST (pl->line, "add")
         || ISINST (pl->line, "cpl")
         || ISINST (pl->line, "dec")
         || ISINST (pl->line, "exg")
@@ -602,11 +749,13 @@ stm8MightRead(const lineNode *pl, const char *what)
         || ISINST (pl->line, "sra")
         || ISINST (pl->line, "srl")
         || ISINST (pl->line, "sub")
-        || ISINST (pl->line, "tnz")))
+        || ISINST (pl->line, "tnz")) &&
+        pl->line[4] == 'a')
           return TRUE;
 
-      if ((pl->line[5] == 'a') && (ISINST (pl->line, "push")
-        || ISINST (pl->line, "swap")))
+      if ((ISINST (pl->line, "push")
+        || ISINST (pl->line, "swap")) &&
+        pl->line[5] == 'a')
           return TRUE;
 
       if ((ISINST (pl->line, "ld") || ISINST (pl->line, "ldf")) && argCont (strchr (pl->line, ','), 'a'))
@@ -617,14 +766,13 @@ stm8MightRead(const lineNode *pl, const char *what)
       if (ISINST (pl->line, "divw") || ISINST (pl->line, "exgw") || ISINST (pl->line, "trap"))
         return TRUE;
  
-      if ((ISINST (pl->line, "exg") && !ISINST (pl->line, "exgw")) && strstr (strchr(pl->line, ','), what))
-        return TRUE;
+      if (ISINST (pl->line, "exg") && strstr (strchr(pl->line, ','), what))
+        return true;
 
-      if (pl->line[4] == extra && ((ISINST (pl->line, "div") || ISINST (pl->line, "mul")) && !ISINST (pl->line, "divw")))
-        return TRUE;
+      if ((ISINST (pl->line, "div") || ISINST (pl->line, "mul")) && pl->line[4] == extra)
+        return true;
 
-      if (pl->line[5] == extra &&
-        (ISINST (pl->line, "addw")
+      if ((ISINST (pl->line, "addw")
         || ISINST (pl->line, "cplw")
         || ISINST (pl->line, "decw")
         || ISINST (pl->line, "incw")
@@ -638,49 +786,53 @@ stm8MightRead(const lineNode *pl, const char *what)
         || ISINST (pl->line, "sraw")
         || ISINST (pl->line, "srlw")
         || ISINST (pl->line, "subw")
-        || ISINST (pl->line, "tnzw")))
+        || ISINST (pl->line, "tnzw")) &&
+        pl->line[5] == extra)
           return TRUE;
 
-      if (pl->line[6] == extra && ISINST (pl->line, "pushw")
-        || ISINST (pl->line, "swapw"))
+      if ((ISINST (pl->line, "pushw")
+        || ISINST (pl->line, "swapw")) && pl->line[6] == extra)
           return TRUE;
 
       if (ISINST (pl->line, "cpw") && pl->line[4] == extra)
         return TRUE;
 
-      if ((strchr (pl->line, ',') ? argCont (strchr (pl->line, ','), extra) : argCont (strchr (pl->line, '('), extra)) && (ISINST (pl->line, "adc")
-        || (ISINST (pl->line, "add") && !ISINST (pl->line, "addw"))
+      if ((strchr (pl->line, ',') ? argCont (strchr (pl->line, ','), extra) : argCont (strchr (pl->line, '('), extra)) &&
+        (ISINST (pl->line, "adc")
+        || ISINST (pl->line, "add")
         || ISINST (pl->line, "and")
         || ISINST (pl->line, "bcp")
         || ISINST (pl->line, "call")
-        || (ISINST (pl->line, "clr") && !ISINST (pl->line, "clrw"))
-        || (ISINST (pl->line, "cp") && !ISINST (pl->line, "cpw"))
+        || ISINST (pl->line, "clr")
+        || ISINST (pl->line, "cp")
         || ISINST (pl->line, "cpl")
-        || (ISINST (pl->line, "dec") && !ISINST (pl->line, "decw"))
-        || (ISINST (pl->line, "inc") && !ISINST (pl->line, "incw"))
+        || ISINST (pl->line, "dec")
+        || ISINST (pl->line, "inc")
         || ISINST (pl->line, "jp")
-        || (ISINST (pl->line, "neg") && !ISINST (pl->line, "negw"))
+        || ISINST (pl->line, "neg")
         || ISINST (pl->line, "or")
-        || (ISINST (pl->line, "rlc") && !ISINST (pl->line, "rlcw"))
-        || (ISINST (pl->line, "rrc") && !ISINST (pl->line, "rrcw"))
+        || ISINST (pl->line, "rlc")
+        || ISINST (pl->line, "rrc")
         || ISINST (pl->line, "sbc")
-        || (ISINST (pl->line, "sll") && !ISINST (pl->line, "sllw"))
-        || (ISINST (pl->line, "sla") && !ISINST (pl->line, "slaw"))
-        || (ISINST (pl->line, "sra") && !ISINST (pl->line, "sraw"))
-        || (ISINST (pl->line, "srl") && !ISINST (pl->line, "srlw"))
-        || (ISINST (pl->line, "sub") && !ISINST (pl->line, "subw"))
-        || (ISINST (pl->line, "swap") && !ISINST (pl->line, "swapw"))
-        || (ISINST (pl->line, "tnz") && !ISINST (pl->line, "tnzw"))
+        || ISINST (pl->line, "sll")
+        || ISINST (pl->line, "sla")
+        || ISINST (pl->line, "sra")
+        || ISINST (pl->line, "srl")
+        || ISINST (pl->line, "sub")
+        || ISINST (pl->line, "swap")
+        || ISINST (pl->line, "tnz")
         || ISINST (pl->line, "cpw")
         || ISINST (pl->line, "ldf")
         || ISINST (pl->line, "ldw")
-        || (ISINST (pl->line, "ld") && !ISINST (pl->line, "ldw"))
+        || ISINST (pl->line, "ld")
         || ISINST (pl->line, "xor")))
           return TRUE;
 
-      if (ISINST (pl->line, "ld"))
+      if (ISINST (pl->line, "ld") || ISINST (pl->line, "ldw"))
         {
-          char buf[64], *p;
+          char buf[128], *p;
+          if (strlen (pl->line) >= 128) // Avoid buffer overflow, err on safe side.
+            return TRUE;
           strcpy (buf, pl->line);
           if (!!(p = strstr (buf, "0x")) || !!(p = strstr (buf, "0X")))
             p[0] = p[1] = ' ';
@@ -689,8 +841,20 @@ stm8MightRead(const lineNode *pl, const char *what)
         }
     }
 
-  if(ISINST(pl->line, "ret"))
-    return(isReturned(what));
+  if (ISINST (pl->line, "call") || ISINST (pl->line, "callr") || ISINST (pl->line, "callf"))
+    {
+      const symbol *f = findSym (SymbolTab, 0, pl->line + (ISINST (pl->line, "call") ? 6 : 7));
+      if (f)
+        return stm8IsParmInCall(f->type, what);
+      else // Fallback needed for calls through function pointers and for calls to literal addresses.
+        return (stm8MightBeParmInCallFromCurrentFunction(what) || stm8MightBeParmInPCallFromCurrentFunction(what));
+    }
+
+  if(ISINST(pl->line, "ret")) // IAR calling convention uses ret for some calls via pointers
+    return(stm8IsReturned(what) || stm8MightBeParmInPCallFromCurrentFunction(what));
+
+  if(ISINST(pl->line, "retf")) // Large model uses retf for calls via function pointers
+    return(stm8IsReturned(what) || stm8MightBeParmInPCallFromCurrentFunction(what));
 
   return FALSE;
 }
@@ -698,27 +862,46 @@ stm8MightRead(const lineNode *pl, const char *what)
 static bool
 stm8UncondJump(const lineNode *pl)
 {
-  return (ISINST(pl->line, "jp\t") || ISINST(pl->line, "jra\t") || ISINST(pl->line, "jpf\t"));
+  return (ISINST(pl->line, "jp") || ISINST(pl->line, "jra") || ISINST(pl->line, "jrt") || ISINST(pl->line, "jpf"));
 }
 
 static bool
 stm8CondJump(const lineNode *pl)
 {
-  return (!stm8UncondJump(pl) && ISINST(pl->line, "jr") ||
-    ISINST(pl->line, "btjt\t") || ISINST(pl->line, "btjf\t"));
+  return (!stm8UncondJump(pl) && STARTSINST(pl->line, "jr") ||
+    ISINST(pl->line, "btjt") || ISINST(pl->line, "btjf"));
 }
 
 static bool
 stm8SurelyWritesFlag(const lineNode *pl, const char *what)
 {
+  if (!strcmp (what, "v") || !strcmp (what, "c"))
+    {        
+      if (ISINST (pl->line, "addw") && !strcmp (pl->line + 5, "sp"))
+        return false;
+      if (ISINST (pl->line, "sub") && !strcmp (pl->line + 4, "sp"))
+        return false;
+
+      if (ISINST (pl->line, "adc") ||
+        STARTSINST (pl->line, "add") || // add, addw
+        STARTSINST (pl->line, "cp") || // cp, cpw, cpl, cplw
+        STARTSINST (pl->line, "div") || // div, divw
+        STARTSINST (pl->line, "neg") || // neg, negw
+        ISINST (pl->line, "sbc") ||
+        STARTSINST (pl->line, "sub")) // sub, subw
+        return true;
+    }
+
   if (!strcmp (what, "n") || !strcmp (what, "z"))
     {
       if (ISINST (pl->line, "addw") && !strcmp (pl->line + 5, "sp"))
-        return FALSE;
+        return false;
       if (ISINST (pl->line, "sub") && !strcmp (pl->line + 4, "sp"))
-        return FALSE;
-      if (ISINST (pl->line, "ld") || ISINST (pl->line, "ldw")) // Todo: Improve accuracy of these two.
-        return FALSE;
+        return false;
+      if (ISINST (pl->line, "ld"))
+        return !stm8InstIsRegToReg(pl->line, false);
+      if (ISINST (pl->line, "ldw"))
+        return !stm8InstIsRegToReg(pl->line, true);
       if (ISINST (pl->line, "pop"))
         return (pl->line[5] == 'c');
       if (ISINST (pl->line, "bccm") || ISINST (pl->line, "bcpl") ||
@@ -729,8 +912,8 @@ stm8SurelyWritesFlag(const lineNode *pl, const char *what)
         ISINST (pl->line, "ccf") ||
         ISINST (pl->line, "exg") || ISINST (pl->line, "exgw") ||
         ISINST (pl->line, "halt") || ISINST (pl->line, "int") ||
-        ISINST (pl->line, "jp") || ISINST (pl->line, "jpf") ||
-        pl->line[0] == 'j' && pl->line[1] == 'r' ||
+        STARTSINST (pl->line, "jp") ||
+        STARTSINST (pl->line, "jr") ||
         ISINST (pl->line, "mov") || ISINST (pl->line, "mul") ||
         ISINST (pl->line, "nop") ||
         ISINST (pl->line, "popw") || ISINST (pl->line, "push") || ISINST (pl->line, "pushw") ||
@@ -738,11 +921,26 @@ stm8SurelyWritesFlag(const lineNode *pl, const char *what)
         ISINST (pl->line, "ret") || ISINST (pl->line, "retf") ||
         ISINST (pl->line, "rvf") || ISINST (pl->line, "scf") ||
         ISINST (pl->line, "sim") || ISINST (pl->line, "trap") || ISINST (pl->line, "wfe") || ISINST (pl->line, "wfi"))
-        return FALSE;
-      return TRUE;
+        return false;
+      return true;
     }
 
-  return FALSE;
+  if (!strcmp (what, "c"))
+    {
+      if (STARTSINST (pl->line, "btj") || // btjt, btjf
+        ISINST (pl->line, "ccf") ||
+        ISINST (pl->line, "rcf") ||
+        STARTSINST (pl->line, "rlc") || // rlc, rlcw
+        STARTSINST (pl->line, "rrc") || // rrc, rrcw
+        ISINST (pl->line, "sbc") ||
+        ISINST (pl->line, "scf") ||
+        STARTSINST (pl->line, "sl") || // sll, sla, sllw, slaw
+        STARTSINST (pl->line, "sr") || // sra, sraw, srl, srlw
+        STARTSINST (pl->line, "sub")) // sub, subw
+        return true;
+    }
+
+  return false;
 }
 
 static bool
@@ -769,8 +967,7 @@ stm8SurelyWrites(const lineNode *pl, const char *what)
         || ISINST (pl->line, "xor"))
           return TRUE;
 
-      if (pl->line[4] == 'a' &&
-        (ISINST (pl->line, "add")
+      if ((ISINST (pl->line, "add")
         || ISINST (pl->line, "clr")
         || ISINST (pl->line, "cpl")
         || ISINST (pl->line, "dec")
@@ -785,13 +982,14 @@ stm8SurelyWrites(const lineNode *pl, const char *what)
         || ISINST (pl->line, "sra")
         || ISINST (pl->line, "srl")
         || ISINST (pl->line, "ldf")
-        || ISINST (pl->line, "sub")))
+        || ISINST (pl->line, "sub")) &&
+        pl->line[4] == 'a')
           return TRUE;
 
-      if (pl->line[5] == 'a' && ISINST (pl->line, "swap"))
+      if (ISINST (pl->line, "swap") && pl->line[5] == 'a')
         return TRUE;
 
-      if (pl->line[3] == 'a' && ISINST (pl->line, "ld"))
+      if (ISINST (pl->line, "ld") && pl->line[3] == 'a')
         return TRUE;
     }
   else
@@ -801,12 +999,13 @@ stm8SurelyWrites(const lineNode *pl, const char *what)
         || ISINST (pl->line, "iret"))
           return TRUE;
 
-      if (pl->line[4] == extra && ((ISINST (pl->line, "div") && !ISINST (pl->line, "divw"))
+      if ((ISINST (pl->line, "div")
         || ISINST (pl->line, "ldw")
-        || ISINST (pl->line, "mul")))
+        || ISINST (pl->line, "mul"))
+        && pl->line[4] == extra)
           return TRUE;
 
-      if (pl->line[5] == extra && (ISINST (pl->line, "addw")
+      if ((ISINST (pl->line, "addw")
         || ISINST (pl->line, "clrw")
         || ISINST (pl->line, "cplw")
         || ISINST (pl->line, "decw")
@@ -821,27 +1020,28 @@ stm8SurelyWrites(const lineNode *pl, const char *what)
         || ISINST (pl->line, "slaw")
         || ISINST (pl->line, "sraw")
         || ISINST (pl->line, "srlw")
-        || ISINST (pl->line, "subw")))
+        || ISINST (pl->line, "subw")) &&
+        pl->line[5] == extra)
           return TRUE;
 
-      if (pl->line[6] == extra && ISINST (pl->line, "swapw"))
+      if (ISINST (pl->line, "swapw") && pl->line[6] == extra)
         return TRUE;
 
-      if ((ISINST (pl->line, "ld") && !ISINST (pl->line, "ldw") && !ISINST (pl->line, "ldf"))
+      if (ISINST (pl->line, "ld")
         && strncmp (pl->line + 3, what, strlen (what)) == 0)
         return TRUE;
 
-      if ((ISINST (pl->line, "exg") && !ISINST (pl->line, "exgw")) && strstr (strstr (pl->line, ","), what))
-        return TRUE;
+      if (ISINST (pl->line, "exg") && strstr (strstr (pl->line, ","), what))
+        return true;
     }
 
-  return FALSE;
+  return false;
 }
 
 static bool
 stm8SurelyReturns(const lineNode *pl)
 {
-  return(ISINST(pl->line, "ret"));
+  return(ISINST(pl->line, "ret") || ISINST(pl->line, "retf"));
 }
 
 /*-----------------------------------------------------------------*/
@@ -893,7 +1093,7 @@ scan4op (lineNode **pl, const char *what, const char *untilOp,
          e.g. isLabel doesn't work there */
       if ((*pl)->isInline)
         {
-          D(("S4O_RD_OP: Inline asm\n"));
+          D(("S4O_ABORT at inline asm\n"));
           return S4O_ABORT;
         }
 
@@ -911,12 +1111,19 @@ scan4op (lineNode **pl, const char *what, const char *untilOp,
           return S4O_RD_OP;
         }
 
+      // Check writes before conditional jumps, some jumps (btjf, btjt) write 'c'
+      if(stm8SurelyWrites(*pl, what))
+        {
+          D(("S4O_WR_OP\n"));
+          return S4O_WR_OP;
+        }
+
       if(stm8UncondJump(*pl))
         {
           *pl = findLabel (*pl);
             if (!*pl)
               {
-                D(("S4O_ABORT\n"));
+                D(("S4O_ABORT at unconditional jump\n"));
                 return S4O_ABORT;
               }
         }
@@ -925,19 +1132,13 @@ scan4op (lineNode **pl, const char *what, const char *untilOp,
           *plCond = findLabel (*pl);
           if (!*plCond)
             {
-              D(("S4O_ABORT\n"));
+              D(("S4O_ABORT at conditional jump\n"));
               return S4O_ABORT;
             }
           D(("S4O_CONDJMP\n"));
           return S4O_CONDJMP;
         }
 
-      if(stm8SurelyWrites(*pl, what))
-        {
-          D(("S4O_WR_OP\n"));
-          return S4O_WR_OP;
-        }
-
       /* Don't need to check for de, hl since stm8MightRead() does that */
       if(stm8SurelyReturns(*pl))
         {
@@ -966,21 +1167,21 @@ doTermScan (lineNode **pl, const char *what)
           case S4O_VISITED:
           case S4O_WR_OP:
             /* all these are terminating conditions */
-            return TRUE;
+            return true;
           case S4O_CONDJMP:
             /* two possible destinations: recurse */
               {
                 lineNode *pl2 = plConditional;
                 D(("CONDJMP trying other branch first\n"));
                 if (!doTermScan (&pl2, what))
-                  return FALSE;
+                  return false;
                 D(("Other branch OK.\n"));
               }
             continue;
           case S4O_RD_OP:
           default:
             /* no go */
-            return FALSE;
+            return false;
         }
     }
 }
@@ -992,7 +1193,7 @@ static void
 unvisitLines (lineNode *pl)
 {
   for (; pl; pl = pl->next)
-    pl->visited = FALSE;
+    pl->visited = false;
 }
 
 bool
@@ -1017,7 +1218,7 @@ stm8notUsedFrom (const char *what, const char *label, lineNode *head)
 {
   lineNode *cpl;
 
-  for (cpl = _G.head; cpl; cpl = cpl->next)
+  for (cpl = head; cpl; cpl = cpl->next)
     if (cpl->isLabel && !strncmp (label, cpl->line, strlen(label)))
       return (stm8notUsed (what, cpl, head));
 
