@@ -5,17 +5,14 @@
 #include <testfwk.h>
 #include <stdint.h>
 
-#if !defined( __SDCC_pdk14) && !defined( __SDCC_pdk15) // Bug #2874
 volatile uint8_t ff = 0xff;
 
 static inline uint16_t and(uint16_t a, uint16_t b) {
   uint16_t r = a & b;
   return r;
 }
-#endif
 
 void testBug(void) {
-#if !defined( __SDCC_pdk14) && !defined( __SDCC_pdk15) // Bug #2874
   goto cond;
 
 cond:
@@ -26,6 +23,5 @@ cond:
 
 end:
   ;
-#endif
 }
 
