@@ -25,7 +25,7 @@ const struct op string_binop2[] = {
 
 void testBug(void)
 {
-#if !defined(__SDCC_mcs51) && !defined(__SDCC_ds390) && !defined(__SDCC_hc08) && !defined(__SDCC_s08) && !defined(__SDCC_pdk14) && !defined(__SDCC_pdk14) && !defined(__SDCC_pdk15) /* mcs51, hc08, s08 and pdk14 have restrictions on function pointers wrt. reentrancy */
+#if !defined(__SDCC_mcs51) && !defined(__SDCC_ds390) && !defined(__SDCC_hc08) && !defined(__SDCC_s08) && !defined(__SDCC_mos6502) && !defined(__SDCC_mos65c02) && !defined(__SDCC_pdk14) && !defined(__SDCC_pdk14) && !defined(__SDCC_pdk15) /* mcs51, hc08, s08 and pdk14 have restrictions on function pointers wrt. reentrancy */
 	ASSERT(((int (*)(long, long))(string_binop1[0].op_func))(1, 1) == 0);
 	ASSERT(((int (*)(long, long))(string_binop1[0].op_func))(1, 2) == 1);
 	ASSERT(((int (*)(long, long))(string_binop2[0].op_func))(1, 1) == 0);
