@@ -4,8 +4,8 @@
 
 #include <testfwk.h>
 
-#if !defined(__SDCC_z80) && !defined(__SDCC_z180) && !defined(__SDCC_r2k) && !defined(__SDCC_r3ka) && !defined(__SDCC_tlcs90) && !defined(__SDCC_ez80_z80)
-#define __z88dk_fastcall
+#ifdef __SDCC
+#pragma disable_warning 85
 #endif
 
 #if !defined(__SDCC_pdk14) && !defined(__SDCC_pdk15) // lack of memory
