@@ -25,17 +25,21 @@ struct S1
 
 struct S1 x = { 1, {{2, 3}, {4, 5}}, {{6, 7}, {8, 9}} };
 
-#if 0 // Enable when SDCC can return struct
+#ifndef __SDCC_mcs51 // mcs51 does not yet support returning struct
+#if !defined(__SDCC_hc08) && !defined(__SDCC_s08) && !defined(__SDCC_mos6502) && !defined(__SDCC_mos65c02) // Bug #3356
+#if !defined(__SDCC_ds390) // Bug #3362
 struct S1 func (void)
 {
   return x;
 }
 #endif
+#endif
+#endif
 
 void
 testTortureExecute (void)
 {
-#if 0
+#if 0 // Enable when sdcc support struct initzalization as below
   struct S1 ret = func ();
 
   if (ret.m2[1].m1 != 9)
